/*	File:	addr_alu.v		  					*/

/*	module name: addr_alu						*/

/*	Description:	The address ALU in the AGU of the DSP core. */


/*	Author: Nitzan Weinberg	, Jonathan Tong		*/


/*==============================================================================*/

// epo set_attribute FIXEDBLOCK = 1


/********************************************************************************/
/*																				*/
/*	module:	addr_alu											*/
/*																				*/
/********************************************************************************/

module addr_alu (
	Rn,
	Nn,
	Mn,
	N_or_1,
	add_sub,
	wrap_once_ena,
	register_new_value
	);


/*==============================================================================*/
/*============	I/O direction	================================================*/
/*==============================================================================*/

parameter DPATH = "addr_alu";

   
input [15:0] Rn;	/* register Rn contents */
input [15:0] Nn;	/* register Nn contents */
input [15:0] Mn;	/* register Mn contents */
input N_or_1;		/* (N_or_1 = 0) => (choose Nn)		*/
			/* (N_or_1 = 1) => (choose 1)	        */

input add_sub;		/* add_sub = 0) => (add Rn with either Nn or 1, depends on N_or_1)	*/
			/* add_sub = 1) => (subtract Rn with either Nn or 1 depends on N_or_1))	*/


input wrap_once_ena;   /* latch control. if 0, the adder/subtracter in module wrap_once around will be disabled */

output [15:0] register_new_value;	/* the result of the new register Rn value. */


/*==============================================================================*/
/*============		internal nets	============================================*/
/*==============================================================================*/

wire [15:0] linear;				/* linear or bitreverse value */

wire [15:0] lower;				/* lower boundary */

wire [15:0] upper;				/* upper boundary */

wire within_boundaries;			/* control signal: is linear within the lower-to-upper range? */

wire [15:0] wrap_once_out;		/* the wrap-once resulted value */

wire [15:0] multi_wrap_out;		/* the multi-wrap resulted value */


wire [1:0] address_mode_select;
						/* (address_mode_select = 2'b00) => (register_new_value = linear)				*/
						/* (address_mode_select = 2'b01) => (register_new_value = wrap_once_addr)		*/
						/* (address_mode_select = 2'b10) => (register_new_value = multi_wrap_addr)		*/

wire in_bound1;
wire in_bound2;
wire out_bound1;
wire out_bound2;

wire wrap_once_add_sub;   /* adder control signal going into wrap_once_address */
/* internal wires for outputs of guard latches */
wire [15:0] wrap_once_in1;
wire [15:0] wrap_once_in2;

/*==============================================================================*/
/*============		Module Instantiation		================================*/
/*==============================================================================*/

/*-----------------------------------------------*/
/* calculates the linear address for register Rn */
/*-----------------------------------------------*/
/*-----------------------------------------------*/
/* calculates bit reverse address for register Rn */
/*-----------------------------------------------*/
linear_bitreverse #(DPATH) linear_bitreverse(
	.Rn (Rn),
	.Nn (Nn),
	.N_or_1 (N_or_1),
	.add_sub (add_sub),
	.address_mode_select (address_mode_select),
	.modified_addr (linear)
	);



/*------------------------------------------------------------------------------*/
/* generates the lower and upper boundries for wrap once modulo addressing mode */
/*------------------------------------------------------------------------------*/

lower_upper lower_upper(
	.Rn (Rn),
	.Mn (Mn),
	.lower (lower),
	.upper (upper)
	);


/*------------------------------------------------------------------------------------*/
/* checks the linear address with compare to the boundries of the modulo addressing.  */
/* Generates control signals if an adjustment is required because of a linear address */
/* that is beyond the boundries range. */
/*------------------------------------------------------------------------------------*/

bound_chk #(DPATH) bound_chk(
	.linear (linear),
	.lower (lower),
	.upper (upper),
	.within_boundaries (within_boundaries)
	);


/*--------------------------------------------------------------------------------------*/
/* calculates the adjusted linear address for a wrap-around-once modulo addressing mode */
/*--------------------------------------------------------------------------------------*/
assign wrap_once_in1 = Mn;
assign wrap_once_in2 = linear;

wire temp;

/* if we're using Nn, the control signal of the wrap_once_address adder/
   subtracter depends on both Nn[15] and add_sub , else it just depends on add_sub*/
wire N_or_1_inv;

`ifdef DW
   assign N_or_1_inv = ~N_or_1;
   assign temp = N_or_1_inv & Nn[15];
   assign wrap_once_add_sub = temp ^~ add_sub;
`else
   inv #(1,0) N_or_1_inverter
      (N_or_1, N_or_1_inv);
   and2 #(1,0) wrap_once_add_control_and
	(N_or_1_inv, Nn[15], temp); 
   xnor2 #(1,0) wrap_once_add_control
	(temp, add_sub, wrap_once_add_sub);
`endif


wrap_once_address #(DPATH) wrap_once_address(
	.linear (wrap_once_in2),
	.Mn (wrap_once_in1),
	.wrap_once_add_sub (wrap_once_add_sub),
	.within_boundaries (within_boundaries),
	.wrap_once_out (wrap_once_out)
	);



/*------------------------------------------------------------------------------------------*/
/* calculates the adjusted linear address for a multiple-wrap-around modulo addressing mode */
/*------------------------------------------------------------------------------------------*/

multi_wrap_address multi_wrap_address(
	.Rn (Rn),
	.Mn (Mn),
	.linear (linear),
	.multi_wrap_out (multi_wrap_out)
	);


/*---------------------------------------------------------------------------------------------------------*/
/* Generates the control signals that choose the addressing mode: linear, wrap-once, multiple-wrap around. */
/*---------------------------------------------------------------------------------------------------------*/

address_mode address_mode(
	.Mn (Mn),
	.address_mode_select (address_mode_select)
	);


/*-------------------------------------------------------------------------------------------------*/
/* selects the correct new value from among the three: linear, wrap-once and multiple-wrap around. */
/*-------------------------------------------------------------------------------------------------*/

`ifdef DW
   DW01_mux_any #(64, 2, 16) output_selector
       ({linear, multi_wrap_out, wrap_once_out, linear}, address_mode_select, register_new_value);  
`else
   mux4  #(16, 1, DPATH) output_selector(
	.IN0 (linear),
	.IN1 (wrap_once_out),
	.IN2 (multi_wrap_out),	
	.IN3 (linear),
	.S0 (address_mode_select[0]),
	.S1 (address_mode_select[1]),
	.Y (register_new_value)
	);
`endif   

endmodule	/* end module addr_alu */
