/*	File:	lower_upper.v	  					*/

/*	module name: lower_upper					*/

/*	Description: Generates the lower and upper boundries for a modulo addressing wrap-around-once mode.	*/


/* Author: Nitzan Weinberg */


/*==============================================================================*/


/********************************************************************************/
/*																				*/
/*	module:	lower_upper															*/
/*																				*/
/********************************************************************************/

module lower_upper (
	Rn,
	Mn,
	lower,
	upper
	);


/*==============================================================================*/
/*============	I/O direction	================================================*/
/*==============================================================================*/

input [15:0] Rn;
input [15:0] Mn;

output [15:0] lower;
output [15:0] upper;



/*==============================================================================*/
/*============		I/O type	================================================*/
/*==============================================================================*/

wire [15:0] Rn;
wire [15:0] Mn;

reg [15:0] lower;
reg [15:0] upper;



/*==============================================================================*/
/*===========	Internal Nets	================================================*/
/*==============================================================================*/



/*==============================================================================*/
/*	Processes																	*/
/*==============================================================================*/

always @(	Rn or
			Mn
		)

	begin	/*------------------*/
		/* wrap around ONCE */
		/*------------------*/

		/* Calculates the lower boundary of the memory region for the modulo operation */
		/* based on the original R address and the most-significant-one in the M register. */
		/* The MS-one determines the number of bits that are zeroed in the R address, */
		/* since the lower address MUST have zeros in its lowest MSBs. */

		/* The upper address of the memory region equals to the lower address plus M, which is */
		/* the same as concatenating the upper portion of Rn with the significant portion of Mn */
		/* (namely, starting with the MS one in Mn).	*/

		if (Mn[14])
			begin
				lower = {Rn[15], 15'h0000};
				upper = {Rn[15], Mn[14:0]};
			end

		else if (Mn[13])
			begin
				lower = {Rn[15:14], 14'h0000};
				upper = {Rn[15:14], Mn[13:0]};
			end

		else if (Mn[12])
			begin
				lower = {Rn[15:13], 13'h0000};
				upper = {Rn[15:13], Mn[12:0]};
			end

		else if (Mn[11])
			begin
				lower = {Rn[15:12], 12'h000};
				upper = {Rn[15:12], Mn[11:0]};
			end

		else if (Mn[10])
			begin
				lower = {Rn[15:11], 11'h000};
				upper = {Rn[15:11], Mn[10:0]};
			end

		else if (Mn[9])
			begin
				lower = {Rn[15:10], 10'h000};
				upper = {Rn[15:10], Mn[9:0]};
			end

		else if (Mn[8])
			begin
				lower = {Rn[15:9], 9'h000};
				upper = {Rn[15:9], Mn[8:0]};
			end

		else if (Mn[7])
			begin
				lower = {Rn[15:8], 8'h00};
				upper = {Rn[15:8], Mn[7:0]};
			end

		else if (Mn[6])
			begin
				lower = {Rn[15:7], 7'h00};
				upper = {Rn[15:7], Mn[6:0]};
			end

		else if (Mn[5])
			begin
				lower = {Rn[15:6], 6'h00};
				upper = {Rn[15:6], Mn[5:0]};
			end

		else if (Mn[4])
			begin
				lower = {Rn[15:5], 5'h00};
				upper = {Rn[15:5], Mn[4:0]};
			end

		else if (Mn[3])
			begin
				lower = {Rn[15:4], 4'h0};
				upper = {Rn[15:4], Mn[3:0]};
			end

		else if (Mn[2])
			begin
				lower = {Rn[15:3], 3'h0};
				upper = {Rn[15:3], Mn[2:0]};
			end

		else if (Mn[1])
			begin
				lower = {Rn[15:2], 2'h0};
				upper = {Rn[15:2], Mn[1:0]};
			end

		else	/* Mn[0]==1 */
			begin
				lower = {Rn[15:1], 1'h0};
				upper = {Rn[15:1], Mn[0]};
			end


	end		/* always */




endmodule		/* end module lower_upper */
