/*   File:	linear_bitreverse.v				*/
/*   module name: linear_bitreverse				*/
/*   Description: Calculates the linear new register contents for the AGU. */
/*   This is a register's new value if either no modulo addressing	*/
/* is required or the new value is within the boundaries of the modulo stack, */
/* as specified by the appropriate M register. */
/* Author: Jonathan Tong		*/

/*===========================================================================*/


/*===========================================================================*/


/********************************************************************************/
/*										*/
/*	module:	linear_bitreverse				      */
/*										*/
/********************************************************************************/

module linear_bitreverse(
	Rn,
	Nn,
	N_or_1,
	add_sub,
	address_mode_select,
	modified_addr
	);



/*============	Parameters	========================================*/

parameter DPATH = "linear_bitreverse";


/*==============================================================================*/
/*============	I/O direction	================================================*/
/*==============================================================================*/

input [15:0] Rn;
input [15:0] Nn;
input N_or_1;  	/* N_or_1 = 0) => (choose Nn)	*/
	      	/* N_or_1 = 1) => (choose 1)	*/
input add_sub;	/* add_sub = 0) => (add Rn with either Nn or 1, depends on N_or_1)  */
		/* add_sub = 1) => (subtract Rn with either Nn or 1 depends on N_or_1))        */
input [1:0] address_mode_select;
output [15:0] modified_addr;  /* the new value for Rn, before any modulo addressing effect */


/*==============================================================================*/
/*============		internal nets		========================================*/
/*==============================================================================*/

wire [15:0] one = 16'h1;		/* constant one */

wire bit_reverse_unbuf;
   
wire bit_reverse;			/* true if it's bit reverse addressing mode */

wire [15:0] internal_N_or_1;			/* the chosen second argument to join Rn */

wire [15:0] add_sub_input1;                     /* 1st input of the 16 bit adder subtracter */
wire [15:0] add_sub_input2;                     /* 2nd input of the 16 bit adder subtracter */
wire [15:0] add_sub_output;                     /* output of the 16 bit adder subtracter */

wire [15:0] reverse_add_sub_output;                     /* reverse output of the 16 bit adder subtracter */

wire [15:0] reverse_Rn;
wire [15:0] reverse_N_or_1;

assign reverse_Rn[15] = Rn[0];
assign reverse_Rn[14] = Rn[1]; 
assign reverse_Rn[13] = Rn[2];
assign reverse_Rn[12] = Rn[3];
assign reverse_Rn[11] = Rn[4];
assign reverse_Rn[10] = Rn[5];
assign reverse_Rn[9] = Rn[6];
assign reverse_Rn[8] = Rn[7];
assign reverse_Rn[7] = Rn[8];
assign reverse_Rn[6] = Rn[9];
assign reverse_Rn[5] = Rn[10];
assign reverse_Rn[4] = Rn[11];
assign reverse_Rn[3] = Rn[12];
assign reverse_Rn[2] = Rn[13];
assign reverse_Rn[1] = Rn[14];
assign reverse_Rn[0] = Rn[15];

/*==============================================================================*/
/*============		module instantiations		================================*/
/*==============================================================================*/

`ifdef DW
   assign bit_reverse = address_mode_select[0] & address_mode_select[1];
`else
   and2 #(1, 0) AND_bitreverse(address_mode_select[0], address_mode_select[1],
		               bit_reverse_unbuf);     
`endif

/*** Buffer added since bit_reverse is drivng 12 inverters ***/   
`ifdef DW
  // Synopsys ignores buffers at this level  
`else    
   stdbuf bit_reverse_buffer (bit_reverse_unbuf, bit_reverse);
`endif

`ifdef DW
   DW01_mux_any #(32, 1, 16) select_N_or_1 
        ({one,Nn}, N_or_1, internal_N_or_1);
`else
   mux2	#(16, 1, DPATH) select_N_or_1
	(
		.IN0 (Nn),
		.IN1 (one),
		.S0 (N_or_1),
		.Y (internal_N_or_1)
	);
`endif
   
 
   
assign reverse_N_or_1[15] = internal_N_or_1[0];
assign reverse_N_or_1[14] = internal_N_or_1[1]; 
assign reverse_N_or_1[13] = internal_N_or_1[2];
assign reverse_N_or_1[12] = internal_N_or_1[3];
assign reverse_N_or_1[11] = internal_N_or_1[4];
assign reverse_N_or_1[10] = internal_N_or_1[5];
assign reverse_N_or_1[9] = internal_N_or_1[6];
assign reverse_N_or_1[8] = internal_N_or_1[7];
assign reverse_N_or_1[7] = internal_N_or_1[8];
assign reverse_N_or_1[6] = internal_N_or_1[9];
assign reverse_N_or_1[5] = internal_N_or_1[10];
assign reverse_N_or_1[4] = internal_N_or_1[11];
assign reverse_N_or_1[3] = internal_N_or_1[12];
assign reverse_N_or_1[2] = internal_N_or_1[13];
assign reverse_N_or_1[1] = internal_N_or_1[14];
assign reverse_N_or_1[0] = internal_N_or_1[15];


// The following muxes select inputs to the adder subtracter 

`ifdef DW
   DW01_mux_any #(32, 1, 16) select_add_sub_input1
   ({reverse_Rn, Rn}, bit_reverse, add_sub_input1);
`else   
  mux2 #(16, 1, DPATH) select_add_sub_input1
	(
		.IN0 (Rn),
		.IN1 (reverse_Rn),
		.S0 (bit_reverse),
		.Y (add_sub_input1)
	);
`endif

`ifdef DW
   DW01_mux_any #(32, 1, 16) select_add_sub_input2
   ({reverse_N_or_1, internal_N_or_1}, bit_reverse, add_sub_input2);
`else   
   mux2 #(16, 1, DPATH) select_add_sub_input2
	(
		.IN0 (internal_N_or_1),
		.IN1 (reverse_N_or_1),
		.S0 (bit_reverse),
		.Y (add_sub_input2)
	);
`endif

`ifdef DW
   DW01_addsub #(16) adder_subtracter(add_sub_input1, add_sub_input2, 1'b0, 
				   add_sub, add_sub_output,);
`else
   add_sub_16bit #(DPATH) adder_subtracter
	(
		.a (add_sub_input1),
		.b (add_sub_input2),
		.control (add_sub),
		.out (add_sub_output)
	);
`endif

// reversing the outputs of the adder/subtracter if bit reversing mode is 
// selected
assign reverse_add_sub_output[15] = add_sub_output[0];
assign reverse_add_sub_output[14] = add_sub_output[1]; 
assign reverse_add_sub_output[13] = add_sub_output[2];
assign reverse_add_sub_output[12] = add_sub_output[3];
assign reverse_add_sub_output[11] = add_sub_output[4];
assign reverse_add_sub_output[10] = add_sub_output[5];
assign reverse_add_sub_output[9] = add_sub_output[6];
assign reverse_add_sub_output[8] = add_sub_output[7];
assign reverse_add_sub_output[7] = add_sub_output[8];
assign reverse_add_sub_output[6] = add_sub_output[9];
assign reverse_add_sub_output[5] = add_sub_output[10];
assign reverse_add_sub_output[4] = add_sub_output[11];
assign reverse_add_sub_output[3] = add_sub_output[12];
assign reverse_add_sub_output[2] = add_sub_output[13];
assign reverse_add_sub_output[1] = add_sub_output[14];
assign reverse_add_sub_output[0] = add_sub_output[15];


`ifdef DW   
  DW01_mux_any #(32, 1, 16) select_output
   ({reverse_add_sub_output, add_sub_output}, bit_reverse, modified_addr);
`else 
   mux2 #(16, 1, DPATH) select_output
	(
		.IN0 (add_sub_output),
		.IN1 (reverse_add_sub_output),
		.S0 (bit_reverse),
		.Y (modified_addr)
	);
`endif

endmodule	/* end module linear_address */





