/*	File:	wrap_once_address.v						*/

/*	module name: wrap_once_address					*/

/*	Description: Generates the new register value for wrap-around once modulo addressing mode.	*/


/* Author: Nitzan Weinberg */


/*==============================================================================*/


/********************************************************************************/
/*																				*/
/*	module:	wrap_once_address													*/
/*																				*/
/********************************************************************************/

module wrap_once_address (
	linear,
	Mn,
	wrap_once_add_sub,
	within_boundaries,
	wrap_once_out
	);

/*============	Parameters	========================================*/

parameter DPATH = "wrap_once_address";

/*==============================================================================*/
/*============	I/O direction	================================================*/
/*==============================================================================*/

input [15:0] linear;
input [15:0] Mn;
input wrap_once_add_sub;
input within_boundaries;

output [15:0] wrap_once_out;



/*==============================================================================*/
/*============		I/O type	================================================*/
/*==============================================================================*/

wire [15:0] linear;				/* the linear address calc by linear_address module */
wire [15:0] Mn;
wire wrap_once_add_sub;			/* (wrap_once_add_sub = 1) => (wrap_once = linear + (Mn + 1)) */
								/* (wrap_once_add_sub = 0) => (wrap_once = linear - (Mn + 1)) */

wire within_boundaries;			/* (within_boundaries = 0) => (wrap_once_addr = wrap_once) */
								/* (within_boundaries = 1) => (wrap_once_addr = linear)    */
								/* if within_boundaries is true, no adjustment to the linear address is required */

wire [15:0] wrap_once_out;		/* the chosen new value, either the linear or the adjusted linear */ 



/*==============================================================================*/
/*===========	Internal Nets	================================================*/
/*==============================================================================*/

wire [15:0] one = 16'h1;	/* constant one */

wire [15:0] Mn_plus_1;		/* Mn+1 */

wire [15:0] wrap_once;		/* the linear address adjusted once with (Mn+1) */
 
wire cout;			/* carry out from the 16 bit adder, not used */
/*==============================================================================*/
/*===========	Module Instantiation	========================================*/
/*==============================================================================*/

/*--------*/
/* Mn + 1 */
/*--------*/

`ifdef DW
   DW01_add #(16) adder_16bit (.A(Mn), .B(one), .CI(1'b0), .SUM(Mn_plus_1), .CO(cout)); 
`else
   addhs #(16,1,DPATH) adder_16bit(
        .A (Mn),
        .B (one),
        .CIN (1'b0),
        .COUT (cout),    /* cout is not connected to anything */
        .SUM (Mn_plus_1)
        );
`endif


/*---------------*/
/* adjust linear */
/*---------------*/
`ifdef DW
   DW01_addsub #(16) adder_subtracter_16bit( .A(linear), .B(Mn_plus_1), 
					     .CI(1'b0), .ADD_SUB(wrap_once_add_sub), 
					     .SUM(wrap_once), .CO() );
`else 
   add_sub_16bit #(DPATH) adder_subtractor_16bit(
	.a (linear),
	.b (Mn_plus_1),
	.control (wrap_once_add_sub),
	.out (wrap_once)
	);
`endif

/*-------------------------------------------------------*/
/* select either the linear or the adjusted linear value */
/*-------------------------------------------------------*/

`ifdef DW
   DW01_mux_any #(32, 1, 16) selector
       ({linear, wrap_once}, within_boundaries, wrap_once_out);
`else   
    mux2  #(16,1,DPATH) selector(
	.IN0 (wrap_once),
	.IN1 (linear),
	.S0 (within_boundaries),
	.Y (wrap_once_out)
	);
`endif



endmodule		/* end module wrap_once_address */
