/*==========================================================
    bs_control:   control logic for bus switch
 ===========================================================*/

`ifdef SIM
   `include "../cmudsp/structural/globals.v"
   `include "../cmudsp/structural/ctl_def.v"
`else
   `include "./verilog/globals.v"
   `include "./verilog/ctl_def.v"
`endif


module bs_control (
		   test,
		   immediate,
		   pdb3,
		   xdb_to_gdb,
		   xdb_to_ydb,
		   ydb_to_gdb,
		   ydb_to_xdb,
		   gdb_to_xdb,
		   gdb_to_ydb,
		   pdb_to_gdb
		   );

   input	    test;
   input	    immediate;
   input [`databus] pdb3;
   
   output	    xdb_to_gdb;
   output	    xdb_to_ydb;
   output	    ydb_to_gdb;
   output	    ydb_to_xdb;
   output	    gdb_to_xdb;
   output	    gdb_to_ydb;
   output	    pdb_to_gdb;
   

   reg		    xdb_to_gdb;
   reg		    xdb_to_ydb;
   reg		    ydb_to_gdb;
   reg		    ydb_to_xdb;
   reg		    gdb_to_xdb;
   reg		    gdb_to_ydb;
   reg		    pdb_to_gdb;

/*----------------------------------------------------------------------------------------------------------*/
/* Decode and carry out the parallel move specified by the move-fieled (pdb[23:8]) in every instruction.	*/
/*----------------------------------------------------------------------------------------------------------*/

   always @( pdb3 or immediate)
   
   begin
      /* initialize the contol signals */
      xdb_to_gdb = `false;
      xdb_to_ydb = `false;
      ydb_to_gdb = `false;
      ydb_to_xdb = `false;
      gdb_to_xdb = `false;
      gdb_to_ydb = `false;
      pdb_to_gdb = `false;

      if (!test) begin          /* disable all tristates in test mode */
      if (!immediate) begin	/* make sure pdb is not immediate data */
      /*---------------------------------*/
      /* Decode the incoming instruction */
      /*---------------------------------*/
      if (pdb3[`movefield] == `no_parallel_data_move)
	 begin	/*--------------------------------------*/
	    /* No parallel data move is required	*/
	    /*			      			*/
	    /* No Action			       */
	    /*--------------------------------------*/
	 end
            
      else if ( {pdb3[23:14], pdb3[7:0]} == 18'b0000_1010_11_1000_0000 )
	 begin	/*--------------------------------------*/
	    /*				JMP Class II			*/
	    /*	No Action				*/
	    /*--------------------------------------*/
	 end		/* JMP Class II */
      else if ( {pdb3[23:14], pdb3[7:0]} == 18'b00000110_11_00100000 )
	 begin	/*--------------------------------------*/
	    /*				REP Class IV			*/
	    /*								*/
	    /* 	Repeat Next Instruction				*/
	    /*--------------------------------------*/
	    // NOT SUPPORTED BY ALU
	    
	    if ( pdb3[13:12] == 2'b00 )		/* from the data ALU - one of the internal registers */
	       case (pdb3[12:8])	/* which is the source */
		 `x0, `x1, `a0, `a1, `a2, `a	:	begin		/* data is on XDB bus */
		    xdb_to_gdb = `true;
		 end
		 `y0, `y1, `b0, `b1, `b2, `b	:	begin		/* data is on YDB bus */
		    ydb_to_gdb = `true;
		 end
		 default						:	;			/* no action */
	       endcase
	    
	 end		/* REP Class IV */
      
      else if ( {pdb3[23:15], pdb3[7], pdb3[5:0]} == 16'b00000110_0_0_100000 )
	 begin	/*--------------------------------------*/
	    /*			REP Class I	or II			*/
	    /*								*/
	    /* Repeat Next Instruction				*/
	    /*--------------------------------------*/
	    
	    if (pdb3[6])
	       begin					/* the number of repetitions is coming from Y memory */
		  ydb_to_gdb = `true;
	       end
	    else
	       begin					/* the number of repetitions is coming from X memory */
		  xdb_to_gdb = `true;
	       end
	 end		/* REP Class I or II */
      else if ( {pdb3[23:16], pdb3[7:4]} == 12'b00000110_1010 )
	 begin	/*--------------------------------------*/
	    /*			REP Class III				*/
	    /*										*/
	    /* No Action							*/
	    /*--------------------------------------*/
	    
	 end		/* REP Class III */
      else if ( {pdb3[23:16], pdb3[7:4]} == 12'b0000_0110_1000 )
	 begin	/*--------------------------------------*/
	    /*			DO Class III		*/
	    /*					        */
	    /* No Action		    	       	*/
	    /*--------------------------------------*/
	 end		/* DO Class III */
      else if ( {pdb3[23:14], pdb3[7:6]} == 12'b00001010_11_00 )
	 begin	/*--------------------------------------*/
	    /*		JCLR / JSET Class III			*/
	    /*							*/
	    /*	No Action				*/
	    /*--------------------------------------*/
	 end		/* JCLR/JSET Class III */
      
      else if ( ({pdb3[23:14], pdb3[7]} == 11'b00001010_01_0) ||
		({pdb3[23:14], pdb3[7], pdb3[5]} == 12'b00001011_01_0_1) )
	 begin	/*--------------------------------------*/
	    /*		BCLR / BSET / BTST Class I		*/
	    /*							*/
	    /*	No Action					*/
	    /*--------------------------------------*/
	 end		/* BCLR/BSET/BTST Class I */
      else if ( ({pdb3[23:14], pdb3[7]} == 11'b00001010_01_1) ||
		({pdb3[23:14], pdb3[7]} == 11'b00001010_00_1) )
	 begin	/*------------------------------------------*/
	    /*		JCLR / JSET Class I	or II		*/
	    /*							*/
	    /*	No Action					*/
	    /*------------------------------------------*/
	 end		/* JCLR/JSET Class I or II */				
      else if ( pdb3[23:13] == 11'b0010_0000_010 )
	 begin	/*----------------------------------*/
	    /*			U:				*/
	    /*						*/
	    /* No Action		 			*/
	    /*----------------------------------*/
	
	 end		/* U */
      	
      else if ( {pdb3[23:16], pdb3[7], pdb3[5]} == 10'b00000101_0_1 )
	 begin	/*----------------------------------*/
	    /*		MOVEC Class I or II			*/
	    /*								*/
	    /* Move Control Register	 		*/
	    /*----------------------------------*/
	   if (pdb3[14])	/* Class I */
	       begin
		  if (pdb3[15])		/* a destination register is written */
		     begin
			if (pdb3[4:3] == 2'b00)		/* M registers */
			   begin
			      if (pdb3[13:8] == 6'b110100)		/* immediate data */
				 begin
				    pdb_to_gdb = `true;
				 end		/* immediate data */
			   end		/* M */
		     end		/* destination */
	       end		/* Class I */ 
	   if (pdb3[15])		/* a destination register is written */
	       begin
		  if (pdb3[4:3] == 2'b00)		/* M registers */
		     begin
			if (pdb3[13:8] != 6'b110100)		/* NOT immediate data */
			   begin
			      if (pdb3[6])
				 ydb_to_gdb = `true;
			      else
				 xdb_to_gdb = `true;
			   end
		     end		/* M */
		  
		  else
		     begin	/* other registers */
		     end
		  
	       end		/* destination */

	    else       	/* a source register is put on the data buses */
	       begin
		  if (pdb3[4:3] == 2'b00)
		     begin	/* M register */
			if (pdb3[6])
			   ydb_to_gdb = `true;
			else
			   xdb_to_gdb = `true;
		     end		/* M */
	       end		/* source */
	    
	 end		/* MOVEC Class I or II */

      else if ( {pdb3[23:17], pdb3[15:14]} == 9'b0000100_00 )
	 begin	/*-------------------------------------------*/
	    /*			X:R Class II  			*/
	    /*							*/
	    /* X Memory and Register Data Move		Class II	*/
	    /*								*/
	    /* Generates the effective address for a data write into X mem. */
	    /* Source and Destination registers are of the Data ALU ONLY.   */
	    /* Needs to put data towards the X data memory.	*/
	    /*--------------------------------------------------------*/
	    
	    // NOT SUPPORTED BY ALU
	 end		/* X:R Class II */
      
      else if ( {pdb3[23:17], pdb3[15:14]} == 9'b0000100_10 )
	 begin	/*---------------------------------------------------*/
	    /*	   	R:Y Class II  				     */
	    /*	      						     */
	    /* Register and Y Memory Data Move		Class II     */
	    /*							     */
	    /* Generates the effective address for a data write into Y mem. */
	    /* Source and Destination registers are of the Data ALU ONLY.   */
	    /* Needs to put data towards the Y data memory.	      */
	    /*--------------------------------------------------------*/
	end
	    
      else if ( pdb3[23:18] == 6'b001000 )
	 begin	/*------------------------------------------*/
	    /*				R:		    */
	    /*		      				*/
	    /* Register to Register Data Move		    */
	    /*						   */
	    /* Active whenever an AGU register is used.	*/
	    /*------------------------------------------*/
	    
	    case ({pdb3[17], pdb3[12]})	/* {source, destination} */
	      2'b00	:	begin
		 /* Source: 		data alu register	*/
		 /* Destination: data alu register	*/
		 case (pdb3[17:13])	/* which is the source */
		   `x0, `x1, `a0, `a1, `a2, `a	:  begin	  /* data is on XDB bus */
		      case (pdb3[12:8])	/* which is the destination */
			`y0, `y1, `b1, `b2, `b:  begin	  /* put data on YDB bus */
			   xdb_to_ydb = `true;
			end
			default:	;			/* no action */
		      endcase
		   end
		   `y0, `y1, `b0, `b1, `b2, `b	:  begin	 /* data is on YDB bus */
		      case (pdb3[12:8])	/* which is the destination */
			`x0, `x1, `a1, `a2, `a :  begin	   /* data put on XDB bus */
			   ydb_to_xdb = `true;
			end
			default:	;			/* no action */
		      endcase
		   end
		   default:	;			/* no action */
		 endcase
		 
	      end
	      2'b01	:	begin
		 /* Source: 		data alu register	*/
		 /* Destination: AGU register		*/
		 case (pdb3[17:13])	/* which is the source */
		   `x0, `x1, `a0, `a1, `a2, `a	:	begin		/* data is on XDB bus */
		      xdb_to_gdb = `true;
		   end
		   `y0, `y1, `b0, `b1, `b2, `b	:	begin		/* data is on YDB bus */
		      ydb_to_gdb = `true;
		   end
		   default:	;			/* no action */
		 endcase
	      end
	      2'b10	:	begin
		 /* Source: 		AGU register		*/
		 /* Destination: data alu register	*/
		 case (pdb3[12:8])	/* which is the destination */
		   `x0, `x1, `a0, `a1, `a2, `a	:	begin		/* data put on XDB bus */
		      gdb_to_xdb = `true;
		      		   end
		   `y0, `y1, `b0, `b1, `b2, `b	:	begin		/* data put on YDB bus */
		      gdb_to_ydb = `true;
		   end
		   default:	;			/* no action */
		 endcase
	      end
	      2'b11	:	begin
		 /* Source: 		AGU register	*/
		 /* Destination: AGU register	*/
		 /* No Action */
	      end
	    endcase		/* {pdb3[17], pdb3[12]} */
	 end		/* R */
      else if ( {pdb3[23:20], pdb3[18]} == 5'b0100_0 )
	 begin	/*----------------------------------*/
	    /* 			   L:					*/
	    /*	No Action						*/
	    /*----------------------------------*/
	 end		/* L: */
      else if ( {pdb3[23:20], pdb3[14]} == 5'b0001_0 )
	 begin	/*----------------------------------------------*/
	    /*		 X:R Class I  			       */
	    /*						*/
	    /* X Memory and Register Data Move		Class I		*/
	    /*							*/
	    /* Generates the effective address for a data read or write. */
	    /* Source and Destination registers are of the Data ALU ONLY.*/
	    /* Needs to get/put data from/towards the X data memory.	 */
	    /*-----------------------------------------------------------*/
	 end		/* X:R Class I */
      
      else if ( {pdb3[23:20], pdb3[14]} == 5'b0001_1 )
	 begin	/*------------------------------------------------*/
	    /*			R:Y Class I  			*/
	    /*							*/
	    /* Register and Y Memory Data Move	Class I		*/
	    /*						*/
	    /* Source and Destination registers are of the Data ALU ONLY. */
	    /* Needs to get/put data from/towards the Y data memory. */
	    /*-----------------------------------------------------*/
	 end		/* R:Y Class I */					
      else if ( pdb3[23:21] == 3'b001 )
	 begin	/*----------------------------------*/
	    /* 			I:			*/
	    /*						*/
	    /* No Action				*/
	    /*----------------------------------*/
	 end		/* I */
      else if ( {pdb3[23:22], pdb3[14]} == 3'b01_1 )
	 begin	/*----------------------------------*/
	    /* 	    X:	or	Y:	Class I		*/
	    /*					*/
	    /* X or Y Memory Data Move			*/
	    /*----------------------------------*/
	    
	  
	    if (pdb3[15:8] == 8'b11_110000)   /* absolute address*/
	       begin
		  if (pdb3[21])	/* the destination for the incoming data is an AGU register */
		     begin
			if (pdb3[19])
			   ydb_to_gdb = `true;
			else
			   xdb_to_gdb = `true;
		     end
		  else	    /* the destination is a data alu register */
		     case ({pdb3[21:20], pdb3[18:16]})
		       `x0, `x1 :  if (pdb3[19])  /* data arriving on YDB */
		       begin
			  ydb_to_xdb = `true;
		       end
		       `y0, `y1	:  if (~pdb3[19])  /* data arriving on XDB */
		       begin
			  xdb_to_ydb = `true;
		       end
		       default:	;	/* no action */
		     endcase
	       end		/* absolute */
	    else if ({pdb3[15],pdb3[13:8]} == 7'b1_110100) /* immediate data */
		     /*--------------------------------------*/
		     /* Special instruction format:	       	*/
		     /* Immediate Data: #xxxxxx, D	Type I	*/
		     /*--------------------------------------*/
	             begin
			if (pdb3[21])	/* the destination is an AGU register */
			      pdb_to_gdb = `true;
		     end
	    else
	       /* for all the other addressing modes:		   */
	       /* 4 cases: read/write from/to X or Y data memories */
	       case ({pdb3[19], pdb3[15]})
		 2'b00 : /* read source register and write the X data memory */
		    case ({pdb3[21:20], pdb3[18:16]})
		      `x0, `x1	:	;	/* no action */
		      `y0, `y1	:	begin
			 ydb_to_xdb = `true;
		      end
		      default:	/* the source is an AGU register */
			 begin
			    //gdb_to_xdb = `true;
			 end
		    endcase
		 2'b01:	/* read X data memory and write a destination register */
		    case ({pdb3[21:20], pdb3[18:16]})
		      `x0, `x1	:	;	/* no action */
		      `y0, `y1	:	begin
			 xdb_to_ydb = `true;
		      end
		      default:	/* the destination is an AGU register */
			 begin
			    //xdb_to_gdb = `true;
			 end
		    endcase
		 2'b10	:	/* read source register and write the Y data memory */
		    case ({pdb3[21:20], pdb3[18:16]})
		      `x0, `x1	:	begin
			 xdb_to_ydb = `true;
		      end
		      `y0, `y1	:	;	/* no action */
		      default						:	/* the source is an AGU register */
			 begin
			    //gdb_to_ydb = `true;
			 end
		    endcase
		 2'b11	:	/* read Y data memory and write a destination register */
		    case ({pdb3[21:20], pdb3[18:16]})
		      `x0, `x1	:	begin
			 ydb_to_xdb = `true;
		      end
		      `y0, `y1	:	;	/* no action */
		      default						:	/* the destination is an AGU register */
			 begin
			    //ydb_to_gdb = `true;
			 end
		    endcase
	       endcase		/* {pdb3[19], pdb3[15]} */
	 end		/* X: or Y: Class I */    	
      else if ( pdb3[23:22] == 2'b01 )
	 begin	/*----------------------------------*/
	    /* 	     X:	or	Y:	Class I     */
	    /*					    */
	    /* X or Y Memory Data Move		    */
	    /*--------------------------------------*/
	    
	    
	 end		/* X: or Y: */
      else if ( pdb3[23] )
	 begin	/*----------------------*/
	    /* 			X:Y:		*/
	    /*						*/
	    /* XY Memory Data Move	*/
	    /*----------------------*/
	 end		/* X:Y: */
      
      else	/* if no operation is required */
	 begin
	 end // else: !if( pdb3[23] )
      end // if (!immediate)
      end // if (!test)
   end	/* immediate */ // always @ ( pdb3 or immediate)


   
endmodule