/*****************************************************************************
 *	File:	core_cascade.v
 *
 *	module name: core_cascade
 *
 *	Description: The CMU DSP core module - comprises of the following units:
 *
 *				 agu_structural 			The address generation unit
 *				 alu_struc		         The data ALU: Ben's model
 *				 bus_switch		         The internal data bus switch
 *				 pcu_structural			The program control unit
 *
 *	Author: Jonathan Ying Fai Tong
 *
 * scan chain: scanin -> AGU -> ALU -> bus switch -> PCU
 ****************************************************************************/

// epoch pre_compiled alu_struc
// epoch pre_compiled agu_structural

`ifdef SIM
    `include "../cmudsp/structural/globals.v"
    `include "../cmudsp/structural/ctl_def.v"
`else
   `include "./verilog/globals.v"
   `include "./verilog/ctl_def.v"
`endif 

module core_cascade (
	reset,
	PDB,
	XDB,
	YDB,
	PAB,
	XAB,
	XWRITE,
	XREAD,
	YAB,
	YWRITE,
	YREAD,
	Clk_agu_0_rfm,
	Clk_agu_1_rfn,
	Clk_agu_2_rfr,
	Clk_agu_3_ctrl,
	Clk_agu_4_ctrl,
	Clk_agu_5_misc,
	Clk_agu_6_misc,
	Clk_alu_0_xy,
	Clk_alu_1_ab,
	Clk_alu_2_misc,
	Clk_bus_0,
	Clk_pcu_0,
	Clk_pcu_1,
	test, scan_in,
	scan_out
	);


/*============	I/O direction	================================================*/

   input		  reset;
   input [`databus]  PDB;
   inout [`databus]  XDB;
   inout [`databus]  YDB;

   inout [`addrbus] PAB;

   output [`addrbus] XAB;
   output		  XWRITE;
   output		  XREAD;

   output [`addrbus] YAB;
   output		  YWRITE;
   output		  YREAD;

   input Clk_agu_0_rfm,
	Clk_agu_1_rfn,
	Clk_agu_2_rfr,
	Clk_agu_3_ctrl,
	Clk_agu_4_ctrl,
	Clk_agu_5_misc,
	Clk_agu_6_misc,
	Clk_alu_0_xy,
	Clk_alu_1_ab,
	Clk_alu_2_misc,
	Clk_bus_0,
	Clk_pcu_0,
	Clk_pcu_1;

   input	     test, scan_in;
   output	     scan_out;

/*=======================================================================*/



/*============		I/O type	=================================*/
   wire		  reset;
   wire [`databus]	  PDB;
   wire [`databus]	  XDB;
   wire [`databus]	  YDB;

   wire [`addrbus]	  PAB;

   wire [`addrbus]	  XAB;
   wire		  XWRITE;
   wire		  XREAD;

   wire [`addrbus]	  YAB;
   wire		  YWRITE;
   wire		  YREAD;

   wire		  test, scan_in;
   wire		  scan_out;

   wire [`databus]        MYPDB;
 
/*========================================================================*/



/*===========	Internal Nets	================================================*/

wire [`databus] GDB;		/* the GDB 24-bit internal bi-directional data bus */

wire [`addrbus] GDB_PCU;	/* GDB connection to the PCU but is 16-bit wide (LSB of the 24 on GDB) */

wire REPEAT;				/* REP instruction signal - generated by PCU */

   wire	      AGU_C;					/* AGU generated carry bit that goes to the data ALU */

   wire [7:0] CCR_from_alu;	/* condition code bits from the data alu */

   wire [7:0] CCR;				/* the CCR register from the PCU */

   wire	      Swrite;				/* S bit (in the CCR) write enable from the data alu */

   wire	      Lwrite;				/* L bit (in the CCR) write enable from the data alu */

   wor	      J;						/* jump bit that comes from either the AGU or the data ALU and goes to the PCU */

   wire	      immediate;                    /* flag for immediate data or absolute address  on PDB */

   wire	      alu_scanout;                  /* scan chain */
   wire       agu_scanout;
   wire       pcu_scanout;
   wire       bus_switch_scanout;

   wire       agu_test;
   wire       alu_test;
   wire       pcu_test;
   wire       bs_test;
   
/*==============================================================================*/

/*===========	Module Instantiation	==================================*/


/*------ scan chain -------*/
//   assign scan_out = bus_switch_scanout;

/*=======================================================================*/   
/*   Twiddle the bits in the incoming instruction                        */
/*=======================================================================*/
   assign MYPDB[3:0] = PDB[23:20];
   assign MYPDB[7:4] = PDB[19:16];
   assign MYPDB[23:8] = PDB[15:0];
    
/*--------------------------*
 *    Split the test signal 
 *    into smaller ones which 
 *   are buffered
 *--------------------------*/
`ifdef DW
   assign agu_test = test;
   assign alu_test = test;
   assign pcu_test = test;
   assign bs_test = test;
`else
   stdbuf agu_test_buf (test, agu_test);
   stdbuf alu_test_buf (test, alu_test);
   stdbuf pcu_test_buf (test, pcu_test);
   stdbuf bs_test_buf (test, bs_test);
`endif

/*--------------------------*/
/*			AGU				*/
/*--------------------------*/

agu_structural agu (
	.reset (reset),
	.PDB (MYPDB),
	.GDB (GDB),
	.REPEAT (REPEAT),
	.PAB (PAB),
	.XAB (XAB),
	.YAB (YAB),
	.XWRITE (XWRITE),
	.XREAD  (XREAD),
	.YWRITE (YWRITE),
	.YREAD  (YREAD),
        .scanin(scan_in),
	.immediate(immediate),		    
	.test(agu_test),
	.scanout(agu_scanout),
	.Clk_0_rfm (Clk_agu_0_rfm),
	.Clk_1_rfn (Clk_agu_1_rfn),
	.Clk_2_rfr (Clk_agu_2_rfr),
	.Clk_3_ctrl (Clk_agu_3_ctrl),
	.Clk_4_ctrl (Clk_agu_4_ctrl),
	.Clk_5_misc (Clk_agu_5_misc),
	.Clk_6_misc (Clk_agu_6_misc)
	);


/*--------------------------*/
/*		data_alu			*/
/*--------------------------*/

alu_struc ALU (
	.reset (reset),
	.PDB (MYPDB),
	.XDB (XDB),
	.YDB (YDB),
	.REPEAT (REPEAT),
	.S1 (S1),
	.S0 (S0),
	.CCR_from_alu (CCR_from_alu),
	.Swrite (Swrite),
	.Lwrite (Lwrite),
	.immediate(immediate),
        .scan_in(agu_scanout),
	.test(alu_test),
	.scan_out(alu_scanout),
	.Clk_0_xy (Clk_alu_0_xy),
	.Clk_1_ab (Clk_alu_1_ab),
	.Clk_2_misc (Clk_alu_2_misc)
	);


/*--------------------------*/
/*		bus_switch			*/
/*--------------------------*/

bus_switch bus_switch (
	.reset (reset),
	.PDB (MYPDB),
	.GDB (GDB),
	.XDB (XDB),
	.YDB (YDB),
	.REPEAT (REPEAT),
	.immediate(immediate),
	.scan_in(alu_scanout), 
	.test(bs_test),
	.scan_out(bus_switch_scanout),
	.Clk (Clk_bus_0)
	);


/*--------------------------*/
/*			PCU				*/
/*--------------------------*/

pcu_structural PCU(
	.reset (reset),
        .immediate(immediate),
	.PDB (MYPDB),
	.GDB (GDB_PCU),
	.PAB (PAB),
	.CCR_from_alu (CCR_from_alu),
	.Swrite (Swrite),
	.Lwrite (Lwrite),
	.J (J),
	.REPEAT (REPEAT),
	.CCR (CCR),
	.S1 (S1),
	.S0 (S0),
        .scan_in(bus_switch_scanout),
	.test(pcu_test),
        .scan_out(scan_out),
	.Clk_0 (Clk_pcu_0),
	.Clk_1 (Clk_pcu_1)
	);


/*==================================================*/


assign GDB_PCU = GDB[`addrbus];		/* the PCU loads only the 16-LSB of the GDB bus */




endmodule	/* core */
