/* $Id: ram_control_signals.v,v 2.4 1998/02/11 18:53:32 benk Exp $ */
/*****************************************************************************
 *
 * ram_control_signals
 *
 * this generates the control signals for the memories.  It takes the twisted
 * signals from the core and corrects them into "usefull" signals for the ram
 * it also takes signals for the external world, uses a decoder to decide
 * which memory is active, and then passes those signals along, (assuming
 * those signals also pass the extra anal test that's built in...)
 *
 *Copyright (C) 1997 Carnegie Mellon University
 *
 *Author: $Author: benk $
 *Date: $Date: 1998/02/11 18:53:32 $
 *Revision: $Revision: 2.4 $
 *
 *
 *Log: $Log: ram_control_signals.v,v $
 *Log: Revision 2.4  1998/02/11 18:53:32  benk
 *Log: Fixed problem with output enable logic
 *Log:
# Revision 2.3  1998/02/10  23:19:40  benk
# Changed logic for ESTROBE, E_XWRITE, and E_YWRITE for writing in DSP mode
#
# Revision 2.2  1998/02/10  21:26:06  inacio
# fixed my typo so that now the right signal gets passed around and this thing
# works
#
# Revision 2.1  1998/02/10  04:37:38  inacio
# This file is the structural version of the ram_enable file.  It controls the
# switching of the control signals to the RAM from the core and the external
# interface.
#
 *****************************************************************************/
module ram_control_signals (cpu_stop, mem_select, clk, cx_write, cx_read,
                           cy_read, cy_write, e_outen, e_write, ep_outen,
                           ep_write, ex_outen, ex_write, ey_outen,
                           ey_write);



   input    cpu_stop;
   input    [1:0]mem_select;
   input    clk;
   input    cx_write;
   input    cx_read;
   input    cy_read;
   input    cy_write;
   input    e_outen;
   input    e_write;
   output   ep_outen;
   output   ep_write;
   output   ex_outen;
   output   ex_write;
   output   ey_outen;
   output   ey_write;


   wire     rcs_ex_write;
   wire     rcs_ex_outen;
   wire     rcs_ey_write;
   wire     rcs_ey_outen;
   wire     rcs_ep_write;
   wire     rcs_ep_outen;
   wire     not_rcs_ex_write;
   wire     not_rcs_ex_outen;
   wire     not_rcs_ey_write;
   wire     not_rcs_ey_outen;
   wire     not_rcs_ep_write;
   wire     not_rcs_ep_outen;
   wire     not_cx_read;
   wire     not_cy_read;
   wire     core_x_write;
   wire     core_y_write;
   wire     external_x_write;
   wire     external_x_outen;
   wire     external_y_write;
   wire     external_y_outen;
   wire     external_p_write;
   wire     external_p_outen;
   

   /*
    *
    * signal conditioning for the signals from the core
    *
    * the read signals are simply inverted and passed on
    *
    * the write signals are inverted and anded with the clock signal to
    * get the correct write pulse
    *
    */
   stdinv cx_read_inv (cx_read, not_cx_read);
   stdinv cy_read_inv (cy_read, not_cy_read);
   stdnand2 core_x_write_and (clk, cx_write, core_x_write);
   stdnand2 core_y_write_and (clk, cy_write, core_y_write);
   



   /*
    * decoder1_3 (in_sig, outx, outy, outp, select);
    * mux2_1 (in_a, in_b, out, select);  (select = 1 -> A  select = 0 -> B)
    *
    */


   /*
    * this decodes the mem_select line and passes through the write/outen
    * signal to the correct mux
    *
    */
   decoder1_3 write_decode (e_write, rcs_ex_write, rcs_ey_write, rcs_ep_write, 
                           mem_select);
   decoder1_3 outen_decode (e_outen, rcs_ex_outen, rcs_ey_outen, rcs_ep_outen,
                           mem_select);


   /*
    *
    * redudant pointless extra logic for safety, the external interface has
    * a million ways of blowing up the interface, but this isn't one of
    * them!
    *
    */
   stdinv rcs_inv_xo (rcs_ex_outen, not_rcs_ex_outen);
   stdnand2 exo_outen_nand (not_rcs_ex_outen, rcs_ex_write, external_x_outen);
   
   stdinv rcs_inv_xw (rcs_ex_write, not_rcs_ex_write);
   stdnand2 exo_write_nand (not_rcs_ex_write, rcs_ex_outen, external_x_write);
   
   stdinv rcs_inv_yo (rcs_ey_outen, not_rcs_ey_outen);
   stdnand2 eyo_outen_nand (not_rcs_ey_outen, rcs_ey_write, external_y_outen);
   
   stdinv rcs_inv_yw (rcs_ey_write, not_rcs_ey_write);
   stdnand2 eyo_write_nand (not_rcs_ey_write, rcs_ey_outen, external_y_write);
   
   stdinv rcs_inv_po (rcs_ep_outen, not_rcs_ep_outen);
   stdnand2 epo_outen_nand (not_rcs_ep_outen, rcs_ep_write, external_p_outen);
   
   stdinv rcs_inv_pw (rcs_ep_write, not_rcs_ep_write);
   stdnand2 epo_write_nand (not_rcs_ep_write, rcs_ep_outen, external_p_write);




   /*
    * the muxes to select the signals and be done with this.
    *
    */
   mux2_1 x_write_mux (core_x_write, external_x_write, ex_write, cpu_stop);
   mux2_1 x_outen_mux (not_cx_read, external_x_outen, ex_outen, cpu_stop);
   mux2_1 y_write_mux (core_y_write, external_y_write, ey_write, cpu_stop);
   mux2_1 y_outen_mux (not_cy_read, external_y_outen, ey_outen, cpu_stop);
   mux2_1 p_write_mux (1'b1, external_p_write, ep_write, cpu_stop);
   mux2_1 p_outen_mux (1'b0, external_p_outen, ep_outen, cpu_stop);


endmodule

/*****************************************************************************
 *****************************************************************************/
