/*	File:	alu_struc.v	  				*/

/*	module name: alu_struv					*/

/*	Description: The data ALU block of the DSP core, 	*/
/*			 "structural" model.	*/

/*  Author:	Nitzan Weinberg, Ben Klass			*/

/* scan chain sequence: scan_in->pdb2->pdb3->conrtol_p->check_imm->conrol->  */
/*                      x0->a2reg->b2reg->b0reg->a0reg->scan_out      */

/*=====================================================================*/





   `include "./verilog/globals.v"


/************************************************************************/
/*									*/
/*	module:	data_alu						*/
/*									*/
/************************************************************************/

module alu_struc (
	reset,
	PDB,
	XDB,
	YDB,
	REPEAT,
//	AGU_C,
	S1,
	S0,
//	CCR,
	CCR_from_alu,
	Swrite,
	Lwrite,
	immediate,
	scan_in, test, scan_out,
	Clk_0_xy,
	Clk_1_ab,
	Clk_2_misc
	);

// epoch set_attribute FIXEDBLOCK = 1
   
/*==========================================================================*/
/*============			I/O direction			============*/
/*==========================================================================*/

   input	    reset;
   input [`databus] PDB;

   inout [`databus] XDB;
   inout [`databus] YDB;

   input	    REPEAT;
//   input	AGU_C;
   input	S1;
   input	S0;
//   input [7:0]	CCR;
   
   input	scan_in, test;
   output	scan_out;

   output [7:0]	CCR_from_alu;
   output	Swrite;
   output	Lwrite;

   input	immediate;

   input	Clk_0_xy;
   input	Clk_1_ab;
   input	Clk_2_misc;



/*==========================================================================*/
/*============				I/O type		============*/
/*==========================================================================*/

   wire		   reset;
   wire [`databus] PDB;
   wire [`databus] XDB;
   wire [`databus] YDB;
   wire		   REPEAT;				/* REP instruction is in progress - coming from the PCU */
// wire	      AGU_C;					/* Carry bit coming from the AGU during BCLR/BSET/BTST inst that concern AGU registers. */
   wire		   S1;					/* scaling bit in MR - from PCU */
   wire		   S0;					/* scaling bit in MR - from PCU */
// wire [7:0] CCR;				/* CCR register coming from the PCU */

   wire [7:0]	   CCR_from_alu;	/* CCR bits generated in the data alu and go to the PCU to be latched in teh CCR regsiter */

   wire		   immediate;		/* flag for immediate data (absolute address extension) on PDB */

   wire		   scan_out;




/*=========================================================================*/
/*===========			Internal Nets		===================*/
/*=========================================================================*/

/* negation of reset because Cascade CLRN is active low */
   wire		   reset_inv;

/* Data lines for parallel moves */

   wire [`databus] ab0, ab1;	/* from a, b registers to parallel move */


   wire [`databus] pdb2_t, pdb2; /* an instruction inside stage 2 (Decode)  */
   wire [`databus] pdb3; /* an instruction inside stage 3 (Execute) */

/* Internal registers */
   wire [`databus] x1, x0;
   wire [`databus] y1, y0;

   wire		   x1_write, x0_write;	/* write enable signal for x registers */
   wire		   y1_write, y0_write;	/* write enable signal for y registers */
   wire		   x1_write_inv, x0_write_inv;	/* negated write enable signal for x */
   wire		   y1_write_inv, y0_write_inv;	/* negated write enable signal for y */
   /* select inputs to XY registers */
   wire [1:0]	   x1_sel, x0_sel;
   wire [1:0]	   y1_sel, y0_sel;

/* Accumulators A and B: 56 bits broken into three fields */
   /* reg_out used before buffers */
   wire [`ext]	   a2, a2reg_out;
   wire [`databus] a1, a0, a0reg_out, a1reg_out;

   wire [`ext]	   b2, b2reg_out;
   wire [`databus] b1, b0, b0reg_out, b1reg_out;

/* register (write) enable signals for A and B */
   wire		   a_write, b_write;		// from data path
   wire		   a0_write, a1_write, a2_write, b0_write, b1_write, b2_write;	// from parallel move
/* NOR of a_write and b_write because Cascade is active low */
   wire		   a0_write_inv, a1_write_inv, a2_write_inv, b0_write_inv, b1_write_inv, b2_write_inv;

/* status register signals.  Sent to PCU to be latched */
   wire		   S_in, L_in, E_in, U_in, N_in, Z_in, V_in, C_in;


/*------------------------------------------------------*/
/*  module interface 					*/
/*  These are the wires that get tied to the modules 	*/
/*------------------------------------------------------*/

   /*----------- Scan Chain -------------*/
   wire		   contol_p_scanout, contol_scanout, x0_scanout;
   

/*-------- Parallel Move ------*/
//   wire		   limit0_lsb, limit1_lsb;
   wire		   limit0_pass, limit1_pass;
   wire		   limit0_l, limit1_l;
   wire [`databus] db2a1,  a1reg_in, a0reg_in, db2b1,  b1reg_in,  b0reg_in;
   wire [`ext]	   a2reg_in, b2reg_in;
 

/* Parallel Move control signals */
   wire		   x0_to_XDB, x1_to_XDB, l0_to_XDB, l1_to_XDB;
   wire		   y0_to_YDB, y1_to_YDB, l0_to_YDB, l1_to_YDB;
   wire		   x0_to_XDB_inv, x1_to_XDB_inv, l0_to_XDB_inv, l1_to_XDB_inv;
   wire		   y0_to_YDB_inv, y1_to_YDB_inv, l0_to_YDB_inv, l1_to_YDB_inv;
   wire		   a0_to_XDB, a0_to_XDB_inv, b0_to_YDB, b0_to_YDB_inv;
   wire		   areg2_sel, breg2_sel;
   wire		   db2a1_sel, db2b1_sel;

/*-------- Data Path -----------*/
   wire [`databus] mult_y, mult_x, mult_xx, mult_yy;
   wire [4:0]	   decode_in;
   wire [`databus] decode_out;
   
   wire [`acc]	   mux_add_out;
   wire [`acc]	   add_in2_sign;	/* in2, negated if subtract */
   wire [`acc]	   add_sum;
   wire [`acc]	   mac_acc, mac_acc_sign_unlatched;

   reg [`acc]	   add_in1, add_in2;
   reg [`acc]      mac_acc_sign;




   wire [`acc]	   mac_out, mult_prod; 	   
   wire		   add_overflow, add_carry, add_zero, add_negative;
   wire		   mac_overflow, mac_carry, mac_zero, mac_negative;
   wire [`acc]	   round_in;
   wire [`acc]	   round_out;
   wire [`acc]	   shift1_in;
   wire [`acc]	   shift1_out;
   wire [`databus] logic_in1, logic_in2;
   wire [`databus] logic_out;
   wire		   ext_not_equal;
/*------------ Control -------------*/
/* These are control inputs to data path modules */
   wire		   add_ena;            /* guard latch enable */
   wire		   add_nadd_sub;  
   wire		   round_s0, round_s1;		/* not hooked to CONTROL */
   wire		   shift1_left, shift1_right;
   wire [2:0]	   logic_ctl;

/*----------------------------------------------*/
/* Data Path control signals 			*/
/*  these are the wires that tell the data path */
/*  modules what they should be hooked up to    */
/*----------------------------------------------*/
/* select lines for Cascade muxes */
   wire [1:0]	   mult_aa_sel, mult_bb_sel;
   wire		   mult_a_sel;
   wire		   mult_ena;            /* guard latch enable */
   wire		   shift1_in_sel;	/* A or B */
   wire		   add_2a_sel;
   wire [1:0]	   add_22c_sel;
   wire [1:0]	   add_21c_sel;
   wire [1:0]	   add_20c_sel;
   wire		   add_2d_sel;
   wire		   round_ctl;           /* 1 if round, 0 if not */
   wire		   logic_in1_sel;
   wire [1:0]	   logic_in2_sel;
   wire		   logic_ena;           /* guard latch enable */
   wire		   ccr_affected;	/* 1 if ccr[5:1] is affected by the operation */


   wire		   Clk_0_xy;
   wire		   Clk_1_ab;
   wire		   Clk_2_misc;

// epoch set_attribute net Clk_0_xy   : NET_SENS = "CLOCK"
// epoch set_attribute net Clk_0_xy   : CLOCKGROUP = 1
// epoch set_attribute net Clk_1_ab   : NET_SENS = "CLOCK"
// epoch set_attribute net Clk_1_ab   : CLOCKGROUP = 1
// epoch set_attribute net Clk_2_misc : NET_SENS = "CLOCK"
// epoch set_attribute net Clk_2_misc : CLOCKGROUP = 1

/*==========================================================================*/
/*==========================================================================*/
/*									*/
/*	Housekeeping							*/
/*									*/
/*==========================================================================*/
/*==========================================================================*/


   assign reset_inv = ~reset;






/*==========================================================================*/
/*==========================================================================*/
/*									*/
/*	Stage 2 (Decode)						*/
/*									*/
/*==========================================================================*/
/*==========================================================================*/

   DW03_reg_s_pl #(24, 0) PDB2_DFF (PDB, Clk_2_misc, reset_inv, ~REPEAT, pdb2);





//   buff#(24, 0) PDB2_BUFF (pdb2_t, pdb2);
   
			
/*==========================================================================*/
/*==========================================================================*/
/*									*/
/*	Stage 3 (Execute)						*/
/*									*/
/*===========================================================================*/
/*===========================================================================*/


   DW03_reg_s_pl #(24, 0) PDB3_DFF (pdb2, Clk_2_misc, reset_inv, 1'b1, pdb3);







/*===========================================================================*/
/*===========			Parallel Move  			============*/
/*===========================================================================*/

/*===========	Module Instantiation	====================================*/

/*--------------------------*/
/*	CONTROL_P	*/
/*--------------------------*/

 control_p CONTROL_P(
	.Clk(Clk_2_misc), .reset(reset),
	.pdb(pdb2),
	.immediate(immediate),
	.x0_to_XDB(x0_to_XDB),
	.x1_to_XDB(x1_to_XDB),
	.y0_to_YDB(y0_to_YDB),
	.y1_to_YDB(y1_to_YDB),
	.l0_to_YDB(l0_to_YDB),
	.l1_to_YDB(l1_to_YDB),
	.b0_to_YDB(b0_to_YDB),
	.l0_to_XDB(l0_to_XDB),
	.l1_to_XDB(l1_to_XDB),
	.a0_to_XDB(a0_to_XDB),
	.x0_write(x0_write),
	.x1_write(x1_write),
	.y0_write(y0_write),
	.y1_write(y1_write),
	.x0_sel(x0_sel),
	.x1_sel(x1_sel),
	.y0_sel(y0_sel),
	.y1_sel(y1_sel),
	.limit0_pass(limit0_pass),
	.limit1_pass(limit1_pass),
	.db2a1_sel(db2a1_sel),
	.areg_sel(areg_sel),	
	.a2_write(a2_write),
	.a1_write(a1_write),
	.a0_write(a0_write),
	.db2b1_sel(db2b1_sel),
	.breg_sel(breg_sel),
	.b2_write(b2_write),
	.b1_write(b1_write),
	.b0_write(b0_write),
		     .scan_in(pdb2[23]), .test(test), 
		     .scan_out(control_p_scanout)
	);

/*--------------------------*/
/*      LIMIT               */
/*--------------------------*/
/* these instatiate Cascade modules */
/* hard wire to A and B registers, cannot do long data moves */
limit_c #("dp24") LIMIT0(
	.in({a2, a1, a0}),
	.S1(S1), .S0(S0),
//	.lsb(limit0_lsb),
	.pass(limit0_pass),
	.out(ab0),
	.L(limit0_l)
	);

limit_c #("dp24") LIMIT1(
	.in({b2, b1, b0}),
	.S1(S1), .S0(S0),
//	.lsb(limit1_lsb),
	.pass(limit1_pass),
	.out(ab1),
	.L(limit1_l)
	);

/*--------------------------*/
/*		XY_REG				*/
/*--------------------------*/
/* these instantiate Cascade modules */

   assign x0_write_inv = ~x0_write;





XYregImm_scan #("dp24") X0_REG(
	.Clk(Clk_0_xy), .reset(reset_inv),
	.DB(XDB), .imm(pdb3[15:8]), .pdb2(pdb2),
	.in_ctl(x0_sel),
	.write(x0_write_inv),
	.scan_in(control_scanout),
	.test(test),
	.scan_out(x0_scanout),
	.q(x0)
	);


   assign x1_write_inv = ~x1_write;





XYregImm_c #("dp24")  X1_REG(
	.Clk(Clk_0_xy), .reset(reset_inv),
	.DB(XDB), .imm(pdb3[15:8]), .pdb2(pdb2),
	.in_ctl(x1_sel),
	.write(x1_write_inv),
	.q(x1)
	);


   assign y0_write_inv = ~y0_write;




   
XYregImm_c #("dp24") Y0_REG(
	.Clk(Clk_0_xy), .reset(reset_inv), .pdb2(pdb2),
	.DB(YDB), .imm(pdb3[15:8]),
	.in_ctl(y0_sel),
	.write(y0_write_inv),
	.q(y0)
	);


   assign y1_write_inv = ~y1_write;





XYregImm_c #("dp24") Y1_REG(
	.Clk(Clk_0_xy), .reset(reset_inv),
	.DB(YDB), .imm(pdb3[15:8]), .pdb2(pdb2),
	.in_ctl(y1_sel),
	.write(y1_write_inv),
	.q(y1)
	);

/*===========	Select data for parallel move to a, b reg	==========*/

/* A inputs */

   DW01_mux_any #(48, 1, 24) MUX_A1REG1  
       ({YDB, XDB}, db2a1_sel, db2a1);  




	
/* B inputs */

   DW01_mux_any #(48, 1, 24) MUX_B1REG1  
       ({YDB, XDB}, db2b1_sel, db2b1);  




	
/*===========	send outputs to busses	===================================*/

/* assign the output data to the XDB/YDB buses if a write enable is true */
/*  possible drivers for XXB port.  Logic ensures only 1 active */	


   assign XDB = x0_to_XDB ? x0 : 24'hz;
   assign XDB = x1_to_XDB ? x1 : 24'hz;
   assign XDB = l0_to_XDB ? ab0 : 24'hz;
   assign XDB = l1_to_XDB ? ab1 : 24'hz;
   assign XDB = a0_to_XDB ? a0 : 24'hz;


























	
/*  possible drivers for YXB port.  Logic ensures only 1 active */

   assign YDB = y0_to_YDB ? y0 : 24'hz;
   assign YDB = y1_to_YDB ? y1 : 24'hz;
   assign YDB = l0_to_YDB ? ab0 : 24'hz;
   assign YDB = l1_to_YDB ? ab1 : 24'hz;
   assign YDB = b0_to_YDB ? b0 : 24'hz;



























/*==========================================================================*/
/*===========			Data Path  		============*/
/*==========================================================================*/

/*===========	Module Instantiation	====================================*/

/*--------------------------*/
/*	CONTROL			*/
/*--------------------------*/
/* this module goes through Synopsys */
control CONTROL (
	.Clk(Clk_2_misc),
	.reset(reset),
	.pdb(pdb2),	
	.immediate(immediate),
	.shift1_in_sel(shift1_in_sel),
	.add_nadd_sub(add_nadd_sub),
	.a_ena(a_write),
	.b_ena(b_write),
	.round_ctl(round_ctl),
	.shift1_left(shift1_left),
	.shift1_right(shift1_right),
	.logic_in1_sel(logic_in1_sel),
	.logic_in2_sel(logic_in2_sel),
	.logic_ena(logic_ena),
	.logic_c(logic_ctl),
	.ccr_affected(ccr_affected),
	.mult_aa_sel(mult_aa_sel),
	.mult_a_sel(mult_a_sel),
	.mult_bb_sel(mult_bb_sel),
	.mult_ena(mult_ena),
	.add_2a_sel(add_2a_sel),
	.add_22c_sel(add_22c_sel), 
	.add_21c_sel(add_21c_sel), 
	.add_20c_sel(add_20c_sel),
	.add_2d_sel(add_2d_sel),
	.add_ena(add_ena),
		     .scan_in(control_p_scanout), .test(test), 
		     .scan_out(control_scanout)
 
	);
   
/*--------------------------*/
/*		MULT	    */
/*--------------------------*/
/* cascade modules for MAC unit */

   assign mac_acc_sign_unlatched = {56{add_nadd_sub}} ^ mac_acc;





/* Guard latches ( x, y latches in MULT_MUX) */

   always@(mult_ena or mac_acc_sign_unlatched)
    if (mult_ena)
     mac_acc_sign <= mac_acc_sign_unlatched;





/* Remember: Multiply A or B by 2 */
/* unit has no carry out, so add extra bit to output */
/* 
hsmult2add #(26, 24, 57, 57) MULT (
        .A({mult_x[23], mult_x, 1'b0}), .B(mult_y), .C({mac_acc_sign[55], mac_acc_sign}),
        .PROD({mac_carry, mac_out})
        );
*/

   DW02_prod_sum1 #(24, 24, 56) MULT(
        .A(mult_x), .B(mult_y), .C({mac_acc_sign[55], mac_acc_sign[55:1]}),
        .TC(1'b1), .SUM({mac_carry, mac_out[55:1]}) 
	);







assign mac_out[0] = mac_acc_sign[0];
   


   assign mult_prod = mac_out ^ {56{add_nadd_sub}};




   
/* if cout is not the same as the sign bit, the MAC overflowed */

   assign mac_overflow = mac_out[55] ^ mac_carry;





/* check for zero */

   assign mac_zero = (mac_out == 56'b0) ? 1'b1 : 1'b0;




   
assign mac_negative = mac_out[55];

   
/*--------------------------*/
/*		DECODE				*/
/*--------------------------*/
/* Synthesized */

   mult_decoder DECODE (decode_in, decode_out);
   

/*--------------------------*/
/*   ADD (subtracts too)    */
/*--------------------------*/
/* contains a cascade module */
/*
 addsub_c ADD (
	.in1(shift1_out),	// do not use mux 
	.in2(add_in2),
	.naddsub(add_nadd_sub),
	.sum(add_sum),
	.overflow(add_overflow),
	.carry(add_carry),
	.zero(add_zero),{
	.negative(add_negative){
	);
*/
	  

/*----------------------*/
/*	ADDER		*/
/*----------------------*/
/* for ADD, SUB operations */
/* use cascade carry look ahead adder */
/* Guard latches  */

   always @(add_ena or shift1_out or mux_add_out)
    if (add_ena)
     begin
	add_in1 <= shift1_out;
        add_in2 <= mux_add_out;
     end    








/* if this is subtract (add_nadd_sub = 1), we need to negate add_in2 */
/* this is done by XORing each bit of add_in2 with add_nadd_sub and setting cin to add_nadd_sub */


   DW01_addsub #(56) ADDER (.A(add_in1), .B(add_in2), .CI(1'b0), .ADD_SUB(add_nadd_sub), .SUM(add_sum), .CO(add_carry)); 








/* if cout is not the same as the sign bit, the adder overflowed */

   assign add_overflow = add_carry ^ add_sum[55];






/* check for zero */

   assign add_zero = (add_sum == 56'b0) ? 1'b1 : 1'b0;





assign add_negative = add_sum[55];
   
/*----------------------*/
/*	ROUND		*/
/*----------------------*/
/* uses Cascade modules and logic */

round_c ROUND (
	.s0(round_s0),.s1(round_s1), 
	.in(round_in),
	.out(round_out)
	);

/*----------------------*/
/*	SHIFT1		*/
/*----------------------*/
/* built with Cascade modules */
shift1_c SHIFT1 (
	.left(shift1_left), .right(shift1_right),
	.in(shift1_in),
	.out(shift1_out),
	.mac_acc(mac_acc)
	);
	
/*----------------------*/
/*	LOGIC		*/
/*----------------------*/
/* built with Cascade modules */
logic_c LOGIC (
	.c(logic_ctl),
	.in1(logic_in1), .in2(logic_in2),
	.out(logic_out)
	);


/*===========	Select module inputs	=============================*/
/* need more inputs tothe always statement */

/* DECODE inputs */
assign decode_in = pdb3[12:8];

/* MULT inputs: mult_x and mult_y */
/* This module has muxes to select inputs to mult_c */

mux_mult MUX_MULT(
	x0, x1, y0, y1,
	decode_out,
	mult_aa_sel, mult_bb_sel,
	mult_a_sel,
	mult_ena,
	mult_xx,
	mult_x, 
	mult_yy,
	mult_y
	);


/* LOGIC inputs */
mux_logic MUX_LOGIC(
	x0, x1, y0, y1,
	a1, b1,
	logic_in1_sel, logic_in2_sel,
	logic_ena,
	logic_in1, logic_in2
	);
	

/* SHIFT1 inputs*/

   DW01_mux_any #(112, 1, 56) MUX_SHIFT1
       ({{b2, b1, b0}, {a2, a1, a0}} , shift1_in_sel, shift1_in);




		
/* ADD inputs*/
/* Select inputs to addsub_c */
mux_add MUX_ADD(
	.logic_out(logic_out),
	.xy1(mult_xx), .xy0(mult_yy),
	.a2(a2), .a1(a1), .a0(a0),
	.b2(b2), .b1(b1), .b0(b0),
	.add_2a_sel(add_2a_sel),
	.add_22c_sel(add_22c_sel), .add_21c_sel(add_21c_sel), .add_20c_sel(add_20c_sel),
	.out(mux_add_out)
	);

	

/* ROUND inputs */
/* Select among adder or mult_prod / sign extend */

   DW01_mux_any #(112, 1, 56) MUX_21
       ({mult_prod, add_sum} , add_2d_sel, round_in);





/* always go through ROUND, use s1, s0 to control if used */

   DW01_mux_any #(4, 1, 2) MUX_ROUND
       ({ {S1, S0}, 2'b11} , round_ctl, {round_s1, round_s0});





/*===========	Latch data into A and B registers	=================*/

/* Data path or parallel move */

   DW01_mux_any #(112, 1, 56) MUX_A_REG2
       ({ {{8{db2a1[23]}}, db2a1, 24'b0}, round_out}, areg_sel, {a2reg_in, a1reg_in, a0reg_in});
   
   assign {a2_write_inv, a1_write_inv, a0_write_inv} = ~({3{a_write}} | {a2_write, a1_write, a0_write});

   DW03_reg_s_pl #(8, 0) A2_DFF(a2reg_in, Clk_2_misc, reset_inv, ~a2_write_inv, a2);

   DW03_reg_s_pl #(24, 0) A1_DFF(a1reg_in, Clk_2_misc, reset_inv, ~a1_write_inv, a1);

   DW03_reg_s_pl #(24, 0) A0_DFF(a0reg_in, Clk_2_misc, reset_inv, ~a0_write_inv, a0);
   



































assign scan_out = a0[23];


/* Data path or parallel move */	

   DW01_mux_any #(112, 1, 56) MUX_B_REG2
       ({ {{8{db2b1[23]}}, db2b1, 24'b0}, round_out}, breg_sel, {b2reg_in, b1reg_in, b0reg_in});
   
   assign {b2_write_inv, b1_write_inv, b0_write_inv} = ~({3{b_write}} | {b2_write, b1_write, b0_write});

   DW03_reg_s_pl #(8, 0) B2_DFF(b2reg_in, Clk_1_ab, reset_inv, ~b2_write_inv, b2);

   DW03_reg_s_pl #(24, 0) B1_DFF(b1reg_in, Clk_1_ab, reset_inv, ~b1_write_inv, b1);

   DW03_reg_s_pl #(24, 0) B0_DFF(b0reg_in, Clk_1_ab, reset_inv, ~b0_write_inv, b0);
   

































/*======================================================================*/
/*									*/
/*	CCR								*/
/*									*/
/*======================================================================*/


/* CCR bits generated in the data alu, and go to the */

assign CCR_from_alu = {S_in, L_in, E_in, U_in, N_in, Z_in, V_in, C_in};

/* these signals are only changed if A or B are source of data move */


   assign L_in = limit0_l | limit1_l;





// signals not used 
assign S_in = 1'b0;
//assign L_in = 1'b0;
assign U_in = 1'b0;
assign Lwrite = 1'b0;
assign Swrite = 1'b0;
   
/* these signals are only changed by ALU instrucitons */ 
/* Should use ccr_affected, but not doing that now */

extcheck_c EXTCHECK(
	.in(round_out[55:46]),
	.S1(S1), .S0(S0),
	.not_equal(ext_no_equal)
	);
assign E_in = ext_no_equal;   

/* these bits come from MAC or adder, use same selector as round unit */

   DW01_mux_any #(8, 1, 4) ZNCV_MUX
       ({ {mac_negative, mac_negative, mac_overflow, mac_carry}, 
          {add_negative, add_negative, add_overflow, add_carry}}, 
          add_2d_sel, {N_in, Z_in, V_in, C_in});






   
   
endmodule /* alu_struc */
			
				



