/*	File:	bus_switch.v	  					        */

/*	module name: bus_switch						        */

/*	Description: The internal DATA bus switch for the CMU DSP core.	*/

/*==============================================================================*/




   `include "./verilog/globals.v"
   `include "./verilog/ctl_def.v"


/************************************************************************/
/*									*/
/*	module:	bus_switch						*/
/*					  			      	*/
/* scan chain: scan_in -> immediate -> pdb2 -> pdb3 -> scan_out         */
/************************************************************************/

module bus_switch (
	reset,
	PDB,
	GDB,
	XDB,
	YDB,
	REPEAT,
	immediate,
	Clk,
	scan_in, test,
	scan_out
	);


/*==============================================================================*/
/*============	I/O direction	================================================*/
/*==============================================================================*/

   input	    reset;
   input [`databus] PDB;

   inout [`databus] GDB;

   inout [`databus] XDB;
   inout [`databus] YDB;

   input	    REPEAT;
   input  immediate;

   input  Clk;

   input  test;
   input  scan_in;
   output scan_out;


/*=========================================================================*/
/*============		I/O type	===================================*/
/*=========================================================================*/

   wire		   reset;
   wire [`databus] PDB;
   wire [`databus] GDB;		/* GDB bus	*/
   wire [`databus] XDB;		/* XDB bus	*/
   wire [`databus] YDB;		/* YDB bus	*/

   wire		   immediate;
		   
   wire		REPEAT;	/* REP instruction signal coming from the PCU */

   wire		   Clk;

   wire		   test;
   wire		   scan_out;


/*========================================================================*/
/*===========	Internal Nets	==========================================*/
/*========================================================================*/

   wire [`databus] pdb2;      /* an instruction inside stage 2 (Decode)	*/
   wire [`databus] pdb3;      /* an instruction inside stage 3 (Execute) */
   wire		   immediate_3;	     /* immediate signal delayedto stage 3 */

   /* enable signals for each set of tristate buffers */
   wire		   xdb_to_gdb;
   wire		   xdb_to_ydb;
   wire		   ydb_to_gdb;
   wire		   ydb_to_xdb;
   wire		   gdb_to_xdb;
   wire		   gdb_to_ydb;
   wire		   pdb_to_gdb;
   

/*-------------------------------------------------------------------*/
/* immediate is a flag  - when true signals that a source's absolute */
/* memory address or an immediate data is used.	*/
/* Such a use required TWO instruction words:   */
/* The instruction and the absolute address/immediate data.	*/
/*---------------------------------------------------------------*/


/*=========================================================================*/
/*									   */
/*	Processes				Processes                  */
/*				  					   */
/*=========================================================================*/

bs_control bs_control(
		   test,
                   immediate_3,
		   pdb3,
		   xdb_to_gdb,
		   xdb_to_ydb,
		   ydb_to_gdb,
		   ydb_to_xdb,
		   gdb_to_xdb,
		   gdb_to_ydb,
		   pdb_to_gdb
	);

/*======================================================================*/
/*	  					 			*/
/*	GDB  XDB  YDB  buses			      			*/
/*								      	*/
/*======================================================================*/

/* assigning data to the GDB, XDB and YDB buses */

   /* enable signals for each set of tristate buffers must be invered*/
   wire		    xdb_to_gdb_inv;
   wire		    xdb_to_ydb_inv;
   wire		    ydb_to_gdb_inv;
   wire		    ydb_to_xdb_inv;
   wire		    gdb_to_xdb_inv;
   wire		    gdb_to_ydb_inv;
   wire		    pdb_to_gdb_inv;


   assign GDB = xdb_to_gdb ? XDB : 24'bz;
   assign GDB = ydb_to_gdb ? YDB : 24'bz;
   assign GDB = pdb_to_gdb ? pdb2 : 24'bz;
   assign XDB = ydb_to_xdb ? YDB : 24'bz;
   assign XDB = gdb_to_xdb ? GDB : 24'bz;
   assign YDB = xdb_to_ydb ? XDB : 24'bz;
   assign YDB = gdb_to_ydb ? GDB : 24'bz;



















/*======================================================================*/
/*				       					*/
/*	Stage 2 (Decode)		    Stage 2	         	*/
/*		 							*/
/*======================================================================*/

wire [`databus] pdb2_in;	
wire reset_inv;

  // assign immediate = 1'b0;




   
assign pdb2_in = (reset) ? `NOP : PDB;


   DW03_reg_s_pl #(24, 0) BS_PDB2_DFF (pdb2_in, Clk, ~reset, ~REPEAT, pdb2); 




/*=================================================================*/
/*	     							   */
/*	Stage 3 (Execute)			       		   */
/*			        				   */
/*=================================================================*/

   DW03_reg_s_pl #(1, 0) IMMEDIATE_3 (immediate, Clk, ~reset, 1'b1, immediate_3);
   DW03_reg_s_pl #(24, 0) BS_PDB3_DFF (pdb2, Clk, ~reset, 1'b1, pdb3); 






assign scan_out = pdb3[23];
       
endmodule	/* bus_switch */
