/*	File:	control.v	  					*/

/*	module name: control						*/

/*	Description: 		 					*/

/*		Control logic for data path				*/

/*		all signals latched					*/


/*======================================================================*/




   `include "./verilog/globals.v"



/************************************************************************/
/*									*/
/*	module:								*/
/*									*/
/************************************************************************/

module control (
	Clk, reset,
	pdb,
	immediate,	
	shift1_in_sel,
	add_nadd_sub,
	a_ena,
	b_ena ,
	round_ctl,
	shift1_left,
	shift1_right,
	logic_in1_sel,
	logic_in2_sel,
	logic_ena,
	logic_c,
	ccr_affected,
	
	mult_aa_sel, mult_a_sel,
	mult_bb_sel,
	mult_ena,
	
	add_2a_sel,
	add_22c_sel, add_21c_sel, add_20c_sel,
	add_2d_sel,
	add_ena,
		
	scan_in, test, scan_out
	);


/*============	I/O direction	==========================================*/

   input	    Clk, reset;
   input [`databus] pdb;	/* instruction word */
   input	    immediate;		/* flag for immediate data on pdb */	

/*------------ Control -------------*/
/*- These are control inputs to data path modules -*/
   output	    add_nadd_sub;  
   output	    shift1_left, shift1_right;
   output [2:0]	    logic_c;
   output	    a_ena, b_ena;

/*----------------------------------------------*/
/* Data Path control signals */
/* these are the wires that tell the modules what */
/* they should be hooked up to */
/*----------------------------------------------*/
   output	    shift1_in_sel;		/* A or B */
   output	    round_ctl;   		/* 1 if round, 0 if not */
   output [1:0]	    logic_in2_sel;
   output	    logic_in1_sel;
   output	    logic_ena;         /* guard latch enable */

   output [1:0]	    mult_aa_sel, mult_bb_sel;
   output	    mult_a_sel;
   output	    mult_ena;         /* guard latch enable */
   
   output	    add_2a_sel;	  /* a or b*/
   output [1:0]	    add_22c_sel;  /* level1, zero, or logic: default to zero */
   output [1:0]	    add_21c_sel;  /* level1, zero, or logic: default to zero */
   output [1:0]	    add_20c_sel;  /* level1, zero, or logic: default to zero */
   output	    add_2d_sel;	  /* level 2 or mult_prod: default to level2 */
   output	    add_ena;      /* guard latch enable signal */
   

/*-------------------- CCR control ------------*/
   output	    ccr_affected;

/*------------- scan chain -----------------*/
   input	    scan_in, test;
   output	    scan_out;
   

/*========================================================================*/


/*============		I/O type	==================================*/


   reg		    add_nadd_sub;  
   reg		    shift1_left, shift1_right;
   reg [2:0]	    logic_c, logic_c_NOT;
   wire		    a_ena, b_ena;

/*----------------------------------------------*/
/* Data Path control signals */
/* these are the wires that tell the modules what */
/* they should be hooked up to */
/* see I/O direction section for signal explanations */
/*----------------------------------------------*/
   reg		    shift1_in_sel;
   reg		    round_ctl;   	
   reg [1:0]	    logic_in2_sel;
   reg		    logic_in1_sel;
   reg		    logic_ena, logic_ena_NOT; 
   
   reg [1:0]	    mult_aa_sel, mult_bb_sel;
   reg		    mult_a_sel;
   reg		    mult_ena, mult_ena_NOT;  

   reg		    add_2a_sel;
   reg [1:0]	    add_22c_sel;
   reg [1:0]	    add_21c_sel;
   reg [1:0]	    add_20c_sel;
   reg		    add_2d_sel;
   reg		    add_ena, add_ena_NOT;
   

/*-------------------- CCR control ------------*/
   reg		    ccr_affected;
    
/*---------------- scan chain -------------------*/
   reg		    scan_out;
 
/*=======================	Internal Signals	=================*/

   reg		    add_nadd_sub_d;  
   reg [1:0]	    shift1_ctl, shift1_ctl_NOT, shift1_ctl_d;	/* {left, right} */
   reg [2:0]	    logic_c_d;
   reg		    a_ena_d, b_ena_d;
   reg		    a_ena_t, b_ena_t;	// before test logic



/*----------------------------------------------*/
/* Data Path control signals */
/* these are the wires that tell the modules what */
/* they should be hooked up to */
/*----------------------------------------------*/
   reg [1:0]	    logic_in2_sel_d;
   reg		    logic_in1_sel_d;
   reg		    logic_ena_d;    

   reg [1:0]	    mult_aa_sel_d;
   reg		    mult_a_sel_d;
   reg [1:0]	    mult_bb_sel_d;
   reg		    mult_ena_d;     

   reg		    add_2a_sel_d;	
   reg [1:0]	    add_22c_sel_d;	
   reg [1:0]	    add_21c_sel_d;	
   reg [1:0]	    add_20c_sel_d;	
   reg		    add_2d_sel_d;	
   reg		    add_ena_d;
   reg		    shift1_in_sel_d;	
   reg		    round_ctl_d;   	
   reg		    legal_instruction;	

/*-------------------- CCR control ------------*/
   reg		    ccr_affected_d;


/*=======================================================================*/

/*===========	Function Declarations	=================================*/


/*----------------------------------------------------------------------*/
/*									*/
/*	function:	legal_inst_complex				*/
/*									*/
/*	Checks if instruction can be an alu instruction			*/
/*	Tries to check data move field for all possible instructions 	*/
/*	that are compatible with alu operations.  			*/
/*	"(masking possible)" means that some "illegal" operations of that */
/*	type are possible,	*/
/*	 e.g. R type MOVE with ddddd = 00000 is not specxified.  */
/*									*/
/*	complex version replaced by simpler legal_inst			*/
/*									*/
/*---------------------------------------------------------------------*/

function legal_inst_complex;

   input [23:0]	pdb;

   begin
      casex (pdb[23:8])
	16'b0010000000000000,	/* No parallel data move */
	16'b001001xxxxxxxxxx, 16'b00101xxxxxxxxxxx, 16'b0011xxxxxxxxxxxx,	/* I */
	16'b001000xxxxxxxxxx,	/* R (masking possible) */
	16'b00100000010xxxxx,	/* U */
	16'b0100x1xxxxxxxxxx, 16'b0101xxxxxxxxxxxx, 16'b011xxxxxxxxxxxx,	/* X:,Y: */
	16'b0001xxxxxxxxxxxx,	/* X:R , R:Y*/
	16'b0000100xx0xxxxxx,	/* X:R, R:Y (masking possible) */
	16'b0100x0xxxxxxxxxx,	/* L: (shared with X:, Y: */
	16'b1xxxxxxxxxxxxxxx:	/* X: Y: */
	   legal_inst_complex = 1;
	16'b00000001000xxxxx:	/* MAC, MACR, MPY, MPYR format 2 */
	   begin
	      if (pdb[7:6] == 2'b11)
		 legal_inst_complex = 1;
	      else
		 legal_inst_complex = 0;
	   end
	default:	legal_inst_complex = 0;
      endcase
   end
   
endfunction


/*----------------------------------------------------------------------*/
/*									*/
/*	function:	legal_inst					*/
/*									*/
/*	Checks if instruction can be an alu instruction			*/
/*	Tries to check data move field for parallel move instructions	*/
/*	Implicit check: all other instuctions begin with 5'b00000	*/
/*	Also check for long arithmatic instuctions: MAC, MACR format II */
/*  Illegal instructions can make it past this check			*/
/*									*/
/*----------------------------------------------------------------------*/

function legal_inst;

   input [23:0]	pdb;
   
   begin
      casex (pdb[23:8])
	16'b00000001000xxxxx:	/* MAC, MACR, MPY, MPYR format 2 */
	   begin
	      if (pdb[7:6] == 2'b11)
		 legal_inst = 1;
	      else
		 legal_inst = 0;
	   end
	16'b0000100xx0xxxxxx:	/* MOVE R:Y, X:R Class II */
	   legal_inst = 1;
	/* any legal long instruction has been covered above, this is default */
	16'b0000xxxxxxxxxxxx:	/* Matches not covered above, not parallel move */
	   legal_inst = 0;
	default:			/* We got this far, must be a parallel move */
	   legal_inst = 1;
      endcase
   end
   
endfunction


/*----------------------------------------------------------------------*/
/*									*/
/*	Decoder functions						*/
/*									*/
/* Instructions which share decode logic for mux selectors use 		*/
/* these functions 							*/
/*									*/
/*----------------------------------------------------------------------*/

/*----------------------------------------------------------------------*/
/*									*/
/* Function:	mult_x_decode						*/
/*									*/
/*	x(a) input to multiplier in format I				*/
/*									*/
/*----------------------------------------------------------------------*/

function [1:0] mult_x_decode;

   input [6:4] pdb;

   begin
      casex (pdb[6:4])
	3'bx00 : begin
	   mult_x_decode = 2'b00;
	end
	3'bx01 : begin
	   mult_x_decode = 2'b01;
	end
	3'bx10 : begin
	   mult_x_decode = 2'b10;
	end
	3'bx11 : begin
	   mult_x_decode = 2'b11;
	end
      endcase
   end
   
endfunction

/*----------------------------------------------------------------------*/
/*									*/
/* Function:	mult_y_decode						*/
/*									*/
/*	y(b) input to multiplier in format I				*/
/*									*/
/*----------------------------------------------------------------------*/

function [1:0] mult_y_decode;

   input [6:4] pdb;

   begin
      casex (pdb[6:4])
	3'b0x0, 3'b101 : begin
	   mult_y_decode = 2'b00;
	end
	3'b0x1, 3'b110 : begin
	   mult_y_decode = 2'b01;
	end
	3'b100 : begin
	   mult_y_decode = 2'b10;
	end
	3'b111 : begin
	   mult_y_decode = 2'b11;
	end
      endcase
   end

endfunction

/*======================================================================*/
/*	processes							*/
/*======================================================================*/

/*===========	Data path cotrol logic	================================*/


always @(pdb or immediate)
   begin

      /* Set default values */
      shift1_in_sel_d = 1'b0;
      add_nadd_sub_d = 0;
      a_ena_d = `false;
      b_ena_d = `false;
      round_ctl_d = 0;
      //	shift1_left_d = 0;
      //	shift1_right_d = 0;
      shift1_ctl_d = 2'b11;	/* default to output zero */
      logic_c_d = 3'b011;
      logic_ena_d = 1'b0;
      ccr_affected_d = 0;
      
      mult_a_sel_d = 1'bx;
      
      add_2a_sel_d 	= 1'bx;		
      add_22c_sel_d	= 2'b00;	
      add_21c_sel_d = 2'b00;	
      add_20c_sel_d	= 2'b00;
      add_2d_sel_d	= 1'b0;	
      add_ena_d = legal_instruction & !pdb[7];
      /* These select lines are decoded for all instructions */
      /* results of the mux may not be used */
      legal_instruction = legal_inst(pdb);
      logic_in2_sel_d = pdb[5:4];
      logic_in1_sel_d = pdb[3];
      mult_aa_sel_d = pdb[5:4];
      mult_bb_sel_d = mult_y_decode(pdb[6:4]);
      mult_ena_d = legal_instruction & pdb[7];
            
      
      /* Check high bits to make sure it is a move */
      if (legal_instruction && !immediate) begin
	 $display($time, " Legal Instruction.");
	 casex (pdb[7:0])	
	   /*--------------------------------------*/
	   /*		CLR			*/
	   /*					*/
	   /* Add:	0 -> D				*/
	   /*--------------------------------------*/
	   /*if ( {pdb[7], pdb[1:0]} == 3'b0_00 )*/
	   8'b0001x011 : begin
	      $display($time, " CLR activated.  PDB = %h", pdb);
	      ccr_affected_d = 1;
	      /* set adder inputs to zero, same as default */
	      add_22c_sel_d	= 1'b0;		/* level1, zero, or logic: set to zero */
	      add_21c_sel_d = 2'b00;	/* level1, zero, or logic: set to zero */
	      add_20c_sel_d	= 2'b00;	/* level1, zero, or logic: set to zero */
	      add_2d_sel_d	= 1'b0;		/* level 2 or mult_prod: default to level2 */
	      /* use shift1 to get zero on add_in1 */
	      shift1_ctl_d = 2'b11;
	      /* acumulator: register to store result */
	      if (pdb[3])
		 begin
		    /* B */
		    a_ena_d = `false;	b_ena_d = `true;  
		 end
	      else begin
		 /* A */
		 a_ena_d = `true;	b_ena_d = `false; 
	      end
	   end /* CLR */	
	   /*--------------------------------------*/
	   /*			ADD, SUB	*/
	   /*					*/
	   /* Add:	D + S -> D			*/
	   /* Add:	D - S -> D			*/
	   /*--------------------------------------*/
	   /*if ( {pdb[7], pdb[1:0]} == 3'b0_00 )*/
	   8'b0xxxxx00 : begin
	      if (pdb[6:4] != 3'b000) begin	
		 $display($time, " ADD, SUB activated.  PDB = %h", pdb);
		 ccr_affected_d = 1;
		 casex (pdb[6:4])	/* source */
		   3'b001	:	begin
		      add_2a_sel_d  = !pdb[3];	/* a or b */
		      add_22c_sel_d = 2'b01;
		      add_21c_sel_d = 2'b10;
		      add_20c_sel_d = 2'b01;
		      add_2d_sel_d  = 1'b0;
		   end
		   3'b01x	: begin	/* Source: X, Y  */
		      add_22c_sel_d = 2'b10;
		      add_21c_sel_d = 2'b11;
		      add_20c_sel_d = 2'b10;
		      add_2d_sel_d  = 1'b0;
		   end
		   3'b1xx	: begin	/* Source: x0, y0, x1, y1 */
		      add_22c_sel_d = 2'b10;
		      add_21c_sel_d = 2'b11;
		      add_20c_sel_d = 2'b00;
		      add_2d_sel_d  = 1'b0;
		   end
		   default	:	;	/* No Action  */
		 endcase	/* pdb[6:4] */
		 /* add or subtract */
		 add_nadd_sub_d = pdb[2];
		 /* acumulator input/output: add_in1 and register to store result */
		 shift1_ctl_d = 2'b00;
		 if (pdb[3])
		    begin
		       /* B */
		       shift1_in_sel_d = 1;
		       a_ena_d = `false;	b_ena_d = `true;  
		    end
		 else begin
		    /* A */
		    shift1_in_sel_d = 0;
		    a_ena_d = `true;	b_ena_d = `false; 
		 end 
		 /* do not round result */
		 round_ctl_d = 0;
	      end /* if */
	   end /* ADD, SUB */
	   /*--------------------------------------*/
	   /*	ADDL, ADDR, SUBL, SUBR		*/
	   /*					*/
	   /* AddR:	D/2 + S -> D		*/
	   /* AddL:	D/2 + S -> D		*/
	   /* SubR:	D*2 - S -> D		*/
	   /* SubL:	D*2 - S -> D		*/
	   /*--------------------------------------*/
	   /*else if ( {pdb[7:5], pdb[1:0]} == 5'b000_10 ) */
	   8'b000xxx10 : begin	
	      $display($time, " ADDL, ADDR, SUBL, SUBR activated.  PDB = %h", pdb);
	      ccr_affected_d = 1;
	      /* add or subtract */
	      add_nadd_sub_d = pdb[2];
	      /* acumulator input/output: add_in1 and register to store result */
	      add_2a_sel_d  = !pdb[3];	/* a or b */
	      add_22c_sel_d = 2'b01;
	      add_21c_sel_d = 2'b10;
	      add_20c_sel_d = 2'b01;
	      add_2d_sel_d  = 1'b0;
	      shift1_ctl_d = 2'b00;
	      if (pdb[3])
		 begin
		    /* B */
		    shift1_in_sel_d = 1;
		    a_ena_d = `false;	b_ena_d = `true;  
		 end
	      else begin
		 /* A */
		 shift1_in_sel_d = 0;
		 a_ena_d = `true;	b_ena_d = `false; 
	      end 
	      /* shift left or right */
	      if (pdb[4])
		 shift1_ctl_d = 2'b10;
	      /*                      shift1_left_d = 1;      */
	      else
		 shift1_ctl_d = 2'b01;
	      /*                      shift1_right_d = 1;     */
	      /* do not round result */
	      round_ctl_d = 0;
	   end /* ADDL, ADDR, SUBL, SUBR */
	   /*--------------------------------------*/
	   /*		ASL, ASR		*/
	   /*					*/
	   /* ASR:	D/2 -> D			*/
	   /* ASL:	D*2 -> D			*/
	   /*--------------------------------------*/
	   /* else if ( {pdb[7:5], pdb[2:0]} == 6'b001_010 ) */
	   8'b001xx010 : begin	
	      $display($time, " ASL, ASR activated.  PDB = %h", pdb);
	      ccr_affected_d = 1;
	      /* set adder_in2 to 0, same as default */
	      add_22c_sel_d = 2'b00;	
	      add_21c_sel_d = 2'b00;	
	      add_20c_sel_d	= 2'b00;
	      add_2d_sel_d = 1'b0;	
	      if (pdb[3])
		 begin
		    /* B */
		    shift1_in_sel_d = 1;
		    a_ena_d = `false;	b_ena_d = `true;  
		 end
	      else begin
		 /* A */
		 shift1_in_sel_d = 0;
		 a_ena_d = `true;	b_ena_d = `false; 
	      end 
	      /* shift left or right */
	      if (pdb[4])
		 shift1_ctl_d = 2'b10;
	      /*			shift1_left_d = 1;	*/
	      else
		 shift1_ctl_d = 2'b01;
	      /*			shift1_right_d = 1;	*/
	      /* do not round result */
	      round_ctl_d = 0;
	   end /* ASL, ASR */
	   /*--------------------------------------*/
	   /*		AND, OR, EOR		*/
	   /*					*/
	   /*		( only D[47:24] )	*/
	   /* AND:	D & S -> D			*/
	   /* OR:	D ^ S -> D			*/
	   /* EOR:	D | S -> D			*/
	   /*--------------------------------------*/
	   /* else if ( {pdb[7:6], pdb[1]} == 6'b01_1 ) */
	   8'b01xxxx1x : begin	
	      if (pdb[2:0] != 3'b111) begin	/* make sure it's not CMPM */
		 $display($time, " AND, OR, EOR activated.  PDB = %h", pdb);
		 ccr_affected_d = 1;
		 /* logic unit input/output: logic_in1 and logic_in2 */
		 /* taken care of in default section */
		 
		 /* logic unit input/output: logic_in1 and register to store result */
		 shift1_ctl_d = 2'b11;	/* use shift1 to output zero */
		 add_2a_sel_d  = pdb[3];
		 add_22c_sel_d = 2'b01;
		 add_21c_sel_d = 2'b01;
		 add_20c_sel_d = 2'b01;
		 add_2d_sel_d  = 1'b0;
		 if (pdb[3])
		    begin
		       /* B */
		       a_ena_d = `false;	b_ena_d = `true;  
		    end
		 else begin
		    /* A */
		    a_ena_d = `true;	b_ena_d = `false; 
		 end 
		 /* operation, leading zero for this operation group */
		 logic_c_d = {1'b0, pdb[2], pdb[0]};
		 /* enable guard latch */
		 logic_ena_d = 1;
		 /* do not round result */
		 round_ctl_d = 0;
	      end
	   end /* AND, OR, EOR */
	   /*--------------------------------------*/
	   /*			NOT		*/
	   /*					*/
	   /*	( only D[47:24] )		*/
	   /* NOT:	~D  -> D			*/
	   /*--------------------------------------*/
	   /* else if ( {pdb[7:6], pdb[1]} == 6'b01_1 ) */
	   8'b0001x111 : begin	
	      $display($time, " NOT activated.  PDB = %h", pdb);
	      ccr_affected_d = 1;
	      /* logic unit input/output: logic_in1 and register to store result */
	      shift1_ctl_d = 2'b11;	/* use shift1 to output zero */
	      add_2a_sel_d  = pdb[3];
	      add_22c_sel_d = 2'b01;
	      add_21c_sel_d = 2'b01;
	      add_20c_sel_d = 2'b01;
	      add_2d_sel_d  = 1'b0;
	      if (pdb[3])
		 begin
		    /* B */
		    a_ena_d = `false;	b_ena_d = `true;  
		 end
	      else begin
		 /* A */
		 a_ena_d = `true;	b_ena_d = `false; 
	      end 
	      /* operation, leading zero for this operation group */
	      logic_c_d = {1'b0, pdb[2], pdb[0]};
	      /* enable guard latch */
	      logic_ena_d = 1;
	      /* do not round result */
	      round_ctl_d = 0;
	   end /* AND, OR, EOR */
	   /*--------------------------------------*/
	   /*	LSL, LSR, ROL, ROR		*/
	   /*					*/
	   /*	( only D[47:24] )		*/
	   /*--------------------------------------*/
	   8'b001xxx11 : begin	
	      $display($time, " LSL, LSR, ROL, ROR activated.  PDB = %h", pdb);
	      ccr_affected_d = 1;
	      /* logic unit input/output: logic_in1 and register to store result */
	      shift1_ctl_d = 2'b11;	/* use shift1 to output zero */
	      add_2a_sel_d  = pdb[3];
	      add_22c_sel_d = 2'b01;
	      add_21c_sel_d = 2'b01;
	      add_20c_sel_d = 2'b01;
	      add_2d_sel_d  = 1'b0;
	      if (pdb[3])
		 begin
		    /* B */
		    a_ena_d = `false;	b_ena_d = `true;  
		 end
	      else begin
		 /* A */
		 a_ena_d = `true;	b_ena_d = `false; 
	      end 
	      /* operation, leading one for this operation group */
	      logic_c_d = {1'b1, pdb[4], pdb[2]};
	      /* enable guard latch */
	      logic_ena_d = 1;
	      /* do not round result */
	      round_ctl_d = 0;
	   end /* LSL, LSR, ROL, ROR */
	   /*--------------------------------------*/
	   /*			MAC		*/
	   /*					*/
	   /* Signed Multiply-Accumulate		*/
	   /*--------------------------------------*/
	   /* else if ( {pdb[7], pdb[1:0]} == 3'b1_10 ) */
	   8'b1xxxxx10 : begin	
	      $display($time, " MAC activated.  PDB = %h", pdb);
	      ccr_affected_d = 1;
	      /* multiplier inputs: mult_x and mult_y */
	      /*--------------------------------------*/
	      /*					*/
	      /* 		MAC Format 1		*/
	      /*					*/
	      /*--------------------------------------*/
	      if ( pdb[23:13] != 11'b00000001_000 )
		 begin
		    /* mult_aa_sel_d, mult_bb_sel_d set globally */
		    mult_a_sel_d = 1;
		 end
	      /*--------------------------------------*/
	      /*					*/
	      /* 	MAC Format 2			*/
	      /*					*/
	      /*--------------------------------------*/
	      else begin
		 /* mult_aa_sel_d, mult_bb_sel_d set globally */
		 mult_a_sel_d  = 0;
		 if (pdb[6] == 0)
		    $display ($time, "ERROR (control): pdb[6] should be 1.  pdb = %h", pdb);
	      end /* else */
	      /*--------------------------------------*/
	      /*					*/
	      /* 		Both formats		*/
	      /*					*/
	      /*--------------------------------------*/
	      /* MULT_result goes to ADD_in2 */
	      add_2d_sel_d = 1'b1;
	      /* add or subtract */
	      add_nadd_sub_d = pdb[2];
	      /* acumulator input/output: add_in1 and register to store result */
	      shift1_ctl_d = 2'b00;
	      if (pdb[3])
		 begin
		    /* B */
		    shift1_in_sel_d = 1;
		    a_ena_d = `false;	b_ena_d = `true;  
		 end
	      else begin
		 /* A */
		 shift1_in_sel_d = 0;
		 a_ena_d = `true;	b_ena_d = `false; 
	      end 
	      /* do not round result */
	      round_ctl_d = 0;
	   end /* MAC  */
	   /*--------------------------------------*/
	   /*			MACR		*/
	   /*					*/
	   /* Signed Multiply-Accumulate round	*/
	   /*--------------------------------------*/
	   /* if ( {pdb[7], pdb[1:0]} == 3'b1_11 )*/
	   8'b1xxxxx11 : begin	
	      $display($time, " MACR activated.  PDB = %h", pdb);
	      ccr_affected_d = 1;
	      /* multiplier inputs: mult_x and mult_y */
	      /*--------------------------------------*/
	      /*					*/
	      /* 		MACR Format 1		*/
	      /*					*/
	      /*--------------------------------------*/
	      if ( pdb[23:13] != 11'b00000001_000 )
		 begin
		    /* mult_aa_sel_d, mult_bb_sel_d set globally */
		    mult_a_sel_d  = 1;
		 end
	      /*--------------------------------------*/
	      /*					*/
	      /* 		MACR Format 2		*/
	      /*					*/
	      /*--------------------------------------*/
	      else begin
		 /* mult_aa_sel_d, mult_bb_sel_d set globally */
		 mult_a_sel_d  = 0;
		 if (pdb[6] == 0)
		    $display ($time, "ERROR (control): pdb[6] should be 1.  pdb = %h", pdb);
	      end /* else */
	      /*--------------------------------------*/
	      /*					*/
	      /* 		Both formats		*/
	      /*					*/
	      /*--------------------------------------*/
	      /* MULT_result goes to ADD_in2 */
	      add_2d_sel_d = 1'b1;
	      /* add or subtract */
	      add_nadd_sub_d = pdb[2];
	      /* acumulator input/output: add_in1 and register to store result */
	      shift1_ctl_d = 2'b00;
	      if (pdb[3])
		 begin
		    /* B */
		    shift1_in_sel_d = 1;
		    a_ena_d = `false;	b_ena_d = `true;  
		 end
	      else begin
		 /* A */
		 shift1_in_sel_d = 0;
		 a_ena_d = `true;	b_ena_d = `false; 
	      end 
	      /* round result */
	      round_ctl_d = 1;
	   end /* MACR  */
	   /*--------------------------------------*/
	   /*		MPY, MPYR		*/
	   /*					*/
	   /* Signed Multiply ( round )		*/
	   /*--------------------------------------*/
	   /* if ( {pdb[7], pdb[1]} == 3'b1_0 ) */
	   8'b1xxxxx0x : begin
	      $display($time, " MPY, MPYR activated.  PDB = %h", pdb);
	      ccr_affected_d = 1;
	      /* multiplier inputs: mult_x and mult_y */
	      /*--------------------------------------*/
	      /*					*/
	      /* 		Format 1		*/
	      /*					*/
	      /*--------------------------------------*/
	      if ( pdb[23:13] != 11'b00000001_000 )
		 begin
		    /* mult_aa_sel_d, mult_bb_sel_d set globally */
		    mult_a_sel_d  = 1;
		 end
	      /*--------------------------------------*/
	      /*					*/
	      /* 		Format 2		*/
	      /*					*/
	      /*--------------------------------------*/
	      else begin
		 /* mult_aa_sel_d, mult_bb_sel_d set globally */
		 mult_a_sel_d  = 0;
		 if (pdb[6] == 0)
		    $display ($time, "ERROR (control): pdb[6] should be 1.  pdb = %h", pdb);
	      end /* else */
	      /*--------------------------------------*/
	      /*					*/
	      /* 		Both formats		*/
	      /*					*/
	      /*--------------------------------------*/
	      /* MULT_result goes to ADD_in2 */
	      add_2d_sel_d = 1'b1;
	      /* add or subtract (from zero)*/
	      add_nadd_sub_d = pdb[2];
	      shift1_ctl_d = 2'b11;	/* use shift1 to output zero */
	      /* acumulator output: register to store result */
	      if (pdb[3])
		 begin
		    /* B */
		    a_ena_d = `false;	b_ena_d = `true;  
		 end
	      else begin
		 /* A */
		 a_ena_d = `true;	b_ena_d = `false; 
	      end 
	      /* round result */
	      round_ctl_d = pdb[0];
	   end /* MPY, MPYR  */
	   /*--------------------------------------*/
	   /*		RND			*/
	   /*					*/
	   /* Round Accumulator			*/
	   /*--------------------------------------*/
	   /* else if ( {pdb[7:4], pdb[2:0]} == 7'b0001_001 ) */
	   8'b0001x001 : begin	
	      $display($time, " RND activated.  PDB = %h", pdb);
	      ccr_affected_d = 1;
	      /* Flow through adder */
	      add_22c_sel_d = 2'b00;		/* level1, zero, or logic: set to zero */
	      add_21c_sel_d = 2'b00;	/* level1, zero, or logic: set to zero */
	      add_20c_sel_d = 2'b00;	/* level1, zero, or logic: set to zero */
	      add_2d_sel_d  = 1'b0;		/* level 2 or mult_prod: default to level2 */
	      shift1_ctl_d = 2'b00;
	      if (pdb[3])
		 begin
		    /* B */
		    shift1_in_sel_d = 1;
		    a_ena_d = `false;	b_ena_d = `true;  
		 end
	      else begin
		 /* A */
		 shift1_in_sel_d = 0;
		 a_ena_d = `true;	b_ena_d = `false; 
	      end 
	      /* round result */
	      round_ctl_d = 1;
	   end /* RND */
	   /*--------------------------------------*/
	   /*		TFR			*/
	   /*					*/
	   /* Transfer data			*/
	   /*--------------------------------------*/
	   8'b0xxxx001 : begin	
	      if (!((pdb[6:4] == 3'b001) || (pdb[6:5] == 2'b01))) begin	/* no aliasing */
		 $display($time, " TFR activated.  PDB = %h", pdb);
		 ccr_affected_d = 1;
		casex (pdb[6:4])	/* source */
		  3'b001	:	begin	/* a or b */
		     add_2a_sel_d  = !pdb[3];
		     add_22c_sel_d = 2'b01;
		     add_21c_sel_d = 2'b10;
		     add_20c_sel_d = 2'b01;
		     add_2d_sel_d  = 1'b0;
		  end
		  3'b01x	: begin	/* Source: X, Y  */
		     add_22c_sel_d = 2'b10;
		     add_21c_sel_d = 2'b11;
		     add_20c_sel_d = 2'b10;
		     add_2d_sel_d  = 1'b0;
		  end
		  3'b1xx	: begin	/* Source: x0, y0, x1, y1 */
		     add_22c_sel_d = 2'b10;
		     add_21c_sel_d = 2'b11;
		     add_20c_sel_d = 2'b00;
		     add_2d_sel_d  = 1'b0;
		  end
		  default	:	;	/* No Action  */
		endcase	/* pdb[6:4] */
		 /* flow through adder */
		 add_nadd_sub_d = 0;
		 shift1_ctl_d = 2'b11;	/* use shift1 to output zero */
		 /* acumulator output: register to store result */
		 if (pdb[3])
		    begin
		       /* B */
		       a_ena_d = `false;	b_ena_d = `true;  
		    end
		 else begin
		    /* A */
		    a_ena_d = `true;	b_ena_d = `false; 
		 end 
		 /* do not round result */
		 round_ctl_d = 0;
	      end /* if */
	   end /* TFR */
	   /******************************/
	   /* Instructions not supported */
	   /******************************/
	   default : 
	      $display ("ERROR (control): Instruction not supported.  PDB = %h", pdb);
	   /* Use default values */
	 endcase
      end	/* if (legal_inst(pdb)) */
   end /* always */
   
   
   
always @(posedge Clk or posedge reset)
   begin
      if (reset)
	 begin
	    /* Set default values */
	    shift1_in_sel = 0;
	    add_nadd_sub = 0;
	    a_ena_t = `false;
	    b_ena_t = `false;
	    round_ctl = 0;
	    //		shift1_left = 0;
	    //		shift1_right = 0;
/* PRESET */
/* PRESET */
	    shift1_ctl_NOT = 2'b00;
/* PRESET */
/* PRESET */
	    logic_c_NOT = 3'b000;
	    ccr_affected = 1'b0;
	    
	    mult_aa_sel = 2'b00;
	    mult_a_sel = 0;
	    mult_bb_sel = 2'b00;
/* PRESET */
	    mult_ena_NOT = 0;      /* enable latch on reset to pass zero */
	    
	    add_2a_sel 	= 0;
	    add_22c_sel	= 0;
	    add_21c_sel = 0;
	    add_20c_sel	= 0;
	    add_2d_sel	= 0;
/* PRESET */
	    add_ena_NOT     = 0;   /* enable latch on reset to pass zero */
	    	    
	    logic_in2_sel = 0;
	    logic_in1_sel = 0;
/* PRESET */
	    logic_ena_NOT     = 0; /* enable latch on reset to pass zero */
	 end
      else begin
	 shift1_in_sel = shift1_in_sel_d;
	 add_nadd_sub = add_nadd_sub_d;
	 a_ena_t =a_ena_d;
	 b_ena_t =b_ena_d;
	 round_ctl = round_ctl_d;
	 //		shift1_left = shift1_left_d;
	 //		shift1_right = shift1_right_d;
	 shift1_ctl_NOT   = ~shift1_ctl_d;
	 logic_c_NOT[2]      = logic_c_d[2];
	 logic_c_NOT[1]      = ~logic_c_d[1];
	 logic_c_NOT[0]      = ~logic_c_d[0];
	 ccr_affected = ccr_affected_d;
	 
	 mult_aa_sel = mult_aa_sel_d;
	 mult_a_sel  = mult_a_sel_d;
	 mult_bb_sel = mult_bb_sel_d;
	 mult_ena_NOT    = ~mult_ena_d;
	 	 
	 add_2a_sel 	= add_2a_sel_d;	
	 add_22c_sel	= add_22c_sel_d;
	 add_21c_sel    = add_21c_sel_d;	
	 add_20c_sel	= add_20c_sel_d;
	 add_2d_sel	= add_2d_sel_d;
	 add_ena_NOT        = ~add_ena_d;
	 
	 logic_in2_sel = logic_in2_sel_d;
	 logic_in1_sel = logic_in1_sel_d;
	 logic_ena_NOT     = ~logic_ena_d;
	 
      end
   end	/* always */
   
   always @(shift1_ctl_NOT)
      {shift1_left, shift1_right} = ~shift1_ctl_NOT;

   always @(logic_c_NOT) begin
      logic_c[2] = logic_c_NOT[2];
      logic_c[1] = ~logic_c_NOT[1];
      logic_c[0] = ~logic_c_NOT[0];
   end

   always @(mult_ena_NOT)
     mult_ena = ~mult_ena_NOT;

   always @(logic_ena_NOT)
     logic_ena = ~logic_ena_NOT;
      
   always @(add_ena_NOT)
     add_ena = ~add_ena_NOT;
      
   /* add gates to disable register enables when test (scan signal) is high */
   assign a_ena = a_ena_t & !test;
   assign b_ena = b_ena_t & !test;
   
   
endmodule	/* end module */
